@@ -121,7 +121,7 @@ bool Annotate::start(const FileHistory* _fh) {
 		ah.insert(*it, FileAnnotation(annId--));
 	while (++it != histRevOrder.constEnd());
 
-	// annotation is splitted in two parts.
+	// annotation is split in two parts.
 	// first we get the list of sha's to feed git diff-tree.
 	// This is very fast.
 	isError = false;
@@ -182,7 +182,7 @@ void Annotate::slotComputeDiffs() {
 void Annotate::annotateFileHistory(SCRef fileName, bool buildPatchScript) {
 
 	// sweep from the oldest to newest so that parent
-	// annotations are calculated before childern
+	// annotations are calculated before children
 	StrVect::const_iterator it(histRevOrder.constEnd());
 	do {
 		--it;
@@ -214,7 +214,7 @@ void Annotate::doAnnotate(SCRef fileName, SCRef sha, bool buildPatchScript) {
 	const QString& parSha = parents.first();
 	FileAnnotation* pa = getFileAnnotation(parSha);
 
-	if (!pa->isValid) {
+	if (!(pa && pa->isValid)) {
 		dbp("ASSERT in doAnnotate: annotation for %1 not valid", parSha);
 		isError = true;
 		return;
@@ -744,6 +744,25 @@ const QString Annotate::getAncestor(SCRef sha, SCRef fileName, int* shaIdx) {
 	return "";
 }
 
+bool Annotate::isDescendant(SCRef sha, SCRef target) {
+// quickly check if target is a direct descendant of sha, i.e. if starting
+// from target, sha could be reached walking along his parents. In case
+// a merge is found the search returns false because you'll need,
+// in general, all the previous ranges to compute the target one.
+
+	const Rev* r = git->revLookup(sha, fh);
+	if (!r)
+		return false;
+
+	int shaIdx = r->orderIdx;
+	r = git->revLookup(target, fh);
+
+	while (r && r->orderIdx < shaIdx && r->parentsCount() == 1)
+		r = git->revLookup(r->parent(0), fh);
+
+	return (r && r->orderIdx == shaIdx);
+}
+
 /*
   Range filtering uses only annotated added/removed lines to compute ranges,
   patch content is never checked, this algorithm is fast but fails in case of
@@ -776,11 +795,14 @@ const QString Annotate::computeRanges(SCRef sha, int rangeStart, int rangeEnd, S
 	// insert starting one, always included by default, could be removed after
 	rangeMap.insert(ancestor, RangeInfo(rangeStart, rangeEnd, true));
 
+	// check if target is a descendant, so to skip back history walking
+	bool isDirectDescendant = isDescendant(ancestor, target);
+
 	// going back in history, to oldest following first parent lane
 	const QString oldest(histRevOrder.last()); // causes a detach!
 	const Rev* curRev = git->revLookup(ancestor, fh); // historyRevs
 	QString curRevSha(curRev->sha());
-	while (curRevSha != oldest) {
+	while (curRevSha != oldest && !isDirectDescendant) {
 
 		if (!diffMap.contains(Key(curRevSha, 0))) {
 			if (curRev->parentsCount() == 0)  // is initial
@@ -818,8 +840,11 @@ const QString Annotate::computeRanges(SCRef sha, int rangeStart, int rangeEnd, S
 	}
 	// now that we have initial revision go back and sweep up all the
 	// remaining stuff, we are guaranteed a good parent is always
-	// found but in case of an indipendent branch, see below
-	for (shaIdx = histRevOrder.count() - 1; shaIdx >= 0; shaIdx--) {
+	// found but in case of an independent branch, see below
+	if (!isDirectDescendant)
+		shaIdx = histRevOrder.count() - 1;
+
+	for ( ; shaIdx >= 0; shaIdx--) {
 
 		SCRef sha(histRevOrder[shaIdx]);
 
@@ -828,7 +853,7 @@ const QString Annotate::computeRanges(SCRef sha, int rangeStart, int rangeEnd, S
 			curRev = git->revLookup(sha, fh);
 
 			if (curRev->parentsCount() == 0) {
-				// the start of an indipendent branch is found in this case
+				// the start of an independent branch is found in this case
 				// insert an empty range, the whole branch will be ignored.
 				// Merge of outside branches are very rare so this solution
 				// seems enough if we don't want to dive in (useless) complications.
@@ -842,6 +867,10 @@ const QString Annotate::computeRanges(SCRef sha, int rangeStart, int rangeEnd, S
 			SCRef parSha(curRev->parent(0));
 
 			if (!rangeMap.contains(parSha)) {
+
+				if (isDirectDescendant) // we must be in a parallel lane, no need
+					continue;       // to compute range info, simply go on
+
 				dbp("ASSERT in rangeFilter: range info for %1 not found", parSha);
 				return "";
 			}
@@ -858,13 +887,13 @@ const QString Annotate::computeRanges(SCRef sha, int rangeStart, int rangeEnd, S
 
 bool Annotate::seekPosition(int* paraFrom, int* paraTo, SCRef fromSha, SCRef toSha) {
 
-	if (paraFrom == 0 || fromSha == toSha)
+	if ((*paraFrom == 0 && *paraTo == 0) || fromSha == toSha)
 		return true;
 
 	QMap<QString, RangeInfo> backup;
 	backup = rangeMap;  // QMap is implicitly shared
 
-	// paragrahps start from 0 but ranges from 1
+	// paragraphs start from 0 but ranges from 1
 	if (computeRanges(fromSha, *paraFrom + 1, *paraTo + 1, toSha).isEmpty())
 		goto fail;
 
