@@ -16,21 +16,34 @@
 #include <qfont.h>
 #include <qvariant.h>
 
+/*
+   QVariant does not support size_t type used in Qt containers, this is
+   a problem on 64bit systems where size_t != uint and when using debug
+   macros on size_t variables, as example dbg(vector.count()), a compile
+   error occurs.
+   Workaround this using a function template and a specialization.
+   Function _valueOf() is used by debug macros
+*/
+template<typename T> inline const QString _valueOf(const T& x) { return QVariant(x).toString(); }
+inline const QString& _valueOf(const QString& x) { return x; }
+inline const QString  _valueOf(size_t x) { return QString::number((uint)x); }
+
 // some debug macros
-#define dbg(x)    qDebug(#x " is <%s>", (QVariant(x)).toString().latin1())
-#define dbs(x)    qDebug((QVariant(x)).toString().latin1())
-#define dbp(s, x) qDebug(QString(s).arg((QVariant(x)).toString()))
-#define db1       qDebug(QString("Mark Nr. 1").latin1())
-#define db2       qDebug(QString("Mark Nr. 2").latin1())
-#define db3       qDebug(QString("Mark Nr. 3").latin1())
-#define db4       qDebug(QString("Mark Nr. 4").latin1())
+#define dbg(x)    qDebug(#x " is <%s>", _valueOf(x).latin1())
+#define dbs(x)    qDebug(_valueOf(x))
+#define dbp(s, x) qDebug(QString(s).arg(_valueOf(x)))
+#define db1       qDebug("Mark Nr. 1")
+#define db2       qDebug("Mark Nr. 2")
+#define db3       qDebug("Mark Nr. 3")
+#define db4       qDebug("Mark Nr. 4")
+#define dbStart   dbs("Starting timer..."); QTime _t; _t.start()
+#define dbRestart dbp("Elapsed time is %1 ms", _t.restart())
 
-// some synctactic sugar
-#define loop(type, x, y) type::const_iterator x(y.constBegin()); \
-                         for (type::const_iterator _e(y.constEnd()); x != _e; ++x)
+// some syntactic sugar
+#define FOREACH(type, i, c) for (type::const_iterator i((c).constBegin()),    \
+                                 _e##i##_((c).constEnd()); i != _e##i##_; ++i)
 
-#define loopList(x, y) QStringList::const_iterator x(y.constBegin()); \
-                       for (QStringList::const_iterator _e(y.constEnd()); x != _e; ++x)
+#define FOREACH_SL(i, c)    FOREACH(QStringList, i, c)
 
 // type shortcuts
 typedef const QString&        SCRef;
@@ -148,6 +161,7 @@ namespace QGit {
 	extern const QColor BROWN;
 	extern const QColor ORANGE;
 	extern const QColor DARK_ORANGE;
+	extern const QColor LIGHT_ORANGE;
 	extern const QColor LIGHT_BLUE;
 	extern const QColor PURPLE;
 	extern const QColor DARK_GREEN;
@@ -161,19 +175,9 @@ namespace QGit {
 	extern const QString PATCHES_DIR;
 	extern const QString PATCHES_NAME;
 
-	// files status
-	extern const QChar MODIFIED;
-	extern const QChar DELETED;
-	extern const QChar NEW;
-	extern const QChar RENAMED;
-	extern const QChar COPIED;
-	extern const QChar UNKNOWN;
-
 	// git index parameters
 	extern const QString ZERO_SHA;
 	extern const QString CUSTOM_SHA;
-	extern const QChar IN_INDEX;
-	extern const QChar NOT_IN_INDEX;
 	extern const QString ALL_MERGE_FILES;
 
 	// settings keys
@@ -230,10 +234,12 @@ namespace QGit {
 
 	// misc helpers
 	bool stripPartialParaghraps(SCRef src, QString* dst, QString* prev);
+	bool writeToFile(SCRef fileName, SCRef data);
+	bool readFromFile(SCRef fileName, QString& data);
 
 	// cache file
 	const uint C_MAGIC  = 0xA0B0C0D0;
-	const int C_VERSION = 12;
+	const int C_VERSION = 13;
 
 	extern const QString BAK_EXT;
 	extern const QString C_DAT_FILE;
@@ -241,35 +247,35 @@ namespace QGit {
 	// misc
 	const int MAX_DICT_SIZE    = 100003; // must be a prime number see QDict docs
 	const int MAX_MENU_ENTRIES = 15;
-	const int MAX_RECENT_REPOS = 5;
+	const int MAX_RECENT_REPOS = 7;
 	extern const QString QUOTE_CHAR;
 }
 
 class Rev {
+	// prevent implicit C++ compiler defaults
+	Rev();
+	Rev(const Rev&);
+	Rev& operator=(const Rev&);
 public:
-	Rev() {}
-	Rev(const QString& data, int idx) : orderIdx(idx), d(data), indexed(false) {
-		isTag = isBranch = isRef = isCurrentBranch = false;
-		isDiffCache = isApplied = isUnApplied = false,
-		descRefsMaster = ancRefsMaster = descBrnMaster = -1;
+	Rev(const QByteArray& b, uint s, int idx, int* next)
+	    : orderIdx(idx), ba(b), start(s) {
 
-		// these fields must be valid from beginning
-		isBoundary = (data[0] == '-'); // public field
-		parentsCnt = (data.find('\n', 39) - (int)isBoundary) / 41;
+		isDiffCache = isApplied = isUnApplied = false;
+		descRefsMaster = ancRefsMaster = descBrnMaster = -1;
+		*next = indexData();
 	}
+	bool isBoundary() const { return (boundaryOfs == 1); }
+	uint parentsCount() const { return parentsCnt; }
 	const QString parent(int idx) const;
 	const QStringList parents() const;
-	const QString sha() const { return isBoundary ? d.mid(1, 40) : d.left(40); }
-	uint parentsCount() const { return parentsCnt; }
-	const QString author() const { init(); return d.mid(autStart, autLen); }
-	const QString authorDate() const { init(); return d.mid(autDateStart, autDateLen); }
-	const QString shortLog() const { init(); return d.mid(sLogStart, sLogLen); }
-	const QString longLog() const { init(); return d.right(lLogLen); }
+	const QString sha() const { return mid(start + boundaryOfs, 40); }
+	const QString author() const { return mid(autStart, autLen); }
+	const QString authorDate() const { return mid(autDateStart, autDateLen); }
+	const QString shortLog() const { return mid(sLogStart, sLogLen); }
+	const QString longLog() const { return mid(lLogStart, lLogLen); }
 
 	QValueVector<int> lanes, childs;
-	QString tag, branch, ref, tagMsg;
-	bool isTag, isBranch, isRef, isCurrentBranch;
-	bool isDiffCache, isApplied, isUnApplied, isBoundary;
+	bool isDiffCache, isApplied, isUnApplied;
 	int orderIdx;
 	QValueVector<int> descRefs;     // list of descendant refs index, normally tags
 	QValueVector<int> ancRefs;      // list of ancestor refs index, normally tags
@@ -279,14 +285,14 @@ public:
 	int descBrnMaster;  // by corresponding index xxxMaster
 
 private:
-	void init() const { if (!indexed) indexData(); }
-	void indexData() const;
+	int indexData();
+	const QString mid(int start, int len) const;
 
-	const QString d;
-	uint parentsCnt;
-	mutable bool indexed;
-	mutable int autStart, autLen, autDateStart;
-	mutable int autDateLen, sLogStart, sLogLen, lLogLen;
+	const QByteArray& ba; // reference here!
+	const uint start;
+	uint parentsCnt, boundaryOfs;
+	int autStart, autLen, autDateStart, autDateLen;
+	int sLogStart, sLogLen, lLogStart, lLogLen;
 };
 typedef QDict<Rev> RevMap; // faster then a map
 
@@ -295,33 +301,53 @@ class RevFile {
 	friend class Cache; // to directly load status
 	friend class Git;
 
-	// 'status' is one ore more comma separated fields. First field
-	// is always the status returned by 'git diff-tree' without -C option.
-	// If file is renamed or copied a second field is added with the status
-	// returned by 'git diff-tree -C' plus source and destination files info.
-	// In case of a working dir file a third field is added with info about
-	// git index status of the file (CACHED_FILE or NO_CACHED_FILE)
-	QValueVector<QString> status;
+	// Status information is splitted in a flags vector and in a string
+	// vector in 'status' are stored flags according to the info returned
+	// by 'git diff-tree' without -C option.
+	// In case of a working dir file an IN_INDEX flag is or-ed togheter in
+	// case file is present in git index.
+	// If file is renamed or copied an entry in 'extStatus' stores the
+	// value returned by 'git diff-tree -C' plus source and destination
+	// files info.
+	// When status of all the files is 'modified' then onlyModified is
+	// set, this let us to do some optimization in this common case
+	bool onlyModified;
+	QValueVector<int> status;
+	QValueVector<QString> extStatus;
 
+	// prevent implicit C++ compiler defaults
+	RevFile(const RevFile&);
+	RevFile& operator=(const RevFile&);
 public:
-	RevFile() {}
+	enum StatusFlag {
+		MODIFIED = 1,
+		DELETED  = 2,
+		NEW      = 4,
+		RENAMED  = 8,
+		COPIED   = 16,
+		UNKNOWN  = 32,
+		IN_INDEX = 64,
+		ANY      = 128
+	};
 
+	RevFile() : onlyModified(true) {}
 	QValueVector<int> dirs; // index of a string vector
 	QValueVector<int> names;
 	QValueVector<int> mergeParent;
 
-	// status helper functions
-	QChar getStatus(int idx) const { return status[idx].at(0); }
-	bool statusCmp(int idx, const QChar& st) const { return status[idx].at(0) == st; }
-	bool isExtendedStatus(int idx) const { return status[idx].length() != 1; }
-	bool isInIndex(int idx) const {
+	// helper functions
+	int count() const { return dirs.count(); }
+	bool statusCmp(int idx, StatusFlag sf) const {
 
-		return (   isExtendedStatus(idx)
-		        && status[idx].section(',', 2, 2) == QGit::IN_INDEX);
+		return ((onlyModified ? MODIFIED : status.at(idx)) & sf);
 	}
-	QString getExtendedStatus(int idx) const {
-
-		return (isExtendedStatus(idx) ? status[idx].section(',', 1, 1) : "");
+	const QString extendedStatus(uint idx) const {
+	/*
+	   rf.extStatus has size equal to position of latest copied/renamed file,
+	   that could be lower then count(), so we have to explicitly check for
+	   an out of bound condition.
+	*/
+		return (!extStatus.isEmpty() && idx < extStatus.count() ? extStatus.at(idx) : "");
 	}
 };
 typedef QDict<RevFile> RevFileMap;
