@@ -20,6 +20,7 @@
 ConsoleImpl::ConsoleImpl(const QString& nm, Git* g) :
              Console(0, 0, Qt::WDestructiveClose), name(nm), git(g) {
 
+	textEditOutput->setFont(QGit::TYPE_WRITER_FONT);
 	QFont f = textLabelCmd->font();
 	f.setBold(true);
 	textLabelCmd->setFont(f);
@@ -73,11 +74,17 @@ bool ConsoleImpl::start(const QString& cmd, const QString& cmdArgs) {
 
 void ConsoleImpl::on_procDataReady(const QString& data) {
 
-	textEditOutput->append(data);
+	QString newParagraph;
+	if (QGit::stripPartialParaghraps(data, &newParagraph, &inpBuf))
+		// QTextEdit::append() adds a new paragraph,
+		// i.e. inserts a LF if not already present.
+		textEditOutput->append(newParagraph);
 }
 
 void ConsoleImpl::on_eof() {
 
+	textEditOutput->append(inpBuf);
+	inpBuf = "";
 	QApplication::restoreOverrideCursor();
 	statusBar()->message("End of \'" + name + "\' execution.");
 	pushButtonStop->setEnabled(false);
