@@ -79,9 +79,7 @@ private:
 	class S {
 	public:
 		S() { clear(); }
-		~S() { clear(); } // FIXME
 		void clear();
-		S& operator=(const S& newState);
 		bool operator==(const S& newState) const;
 		bool operator!=(const S& newState) const;
 
@@ -107,7 +105,7 @@ public:
 	void setThrowOnDelete(bool b);
 	bool isThrowOnDeleteRaised(int excpId, SCRef curContext);
 	MainImpl* m() const;
-	QString dragHostName() const;
+	const QString dragHostName() const;
 	bool isReadyToDrag() const { return readyToDrag; }
 	bool setReadyToDrag(bool b);
 	bool isDragging() const { return dragging; }
@@ -150,6 +148,7 @@ private:
 	void sendPopupEvent();
 
 	EM_DECLARE(exDeleteRequest);
+	EM_DECLARE(exCancelRequest);
 
 	bool readyToDrag;
 	bool dragging;
