@@ -176,7 +176,7 @@ void FileContent::update(bool force) {
 		// call seekPosition() while loading the file so to shadow the compute time
 		int& from = ss.hasSelectedText ? ss.paraFrom : ss.topPara;
 		int& to = ss.hasSelectedText ? ss.paraTo : ss.topPara;
-		ss.isValid = (annotateObj->seekPosition(&from, &to, st->sha(false), st->sha(true)));
+		ss.isValid = annotateObj->seekPosition(&from, &to, st->sha(false), st->sha(true));
 	}
 }
 
@@ -188,7 +188,7 @@ void FileContent::startAnnotate(FileHistory* fh) {
 uint FileContent::annotateLength(const FileAnnotation* annFile) {
 
 	uint maxLen = 0;
-	loopList(it, annFile->lines)
+	FOREACH_SL (it, annFile->lines)
 		if ((*it).length() > maxLen)
 			maxLen = (*it).length();
 
@@ -213,7 +213,7 @@ void FileContent::goToAnnotation(int revId) {
 
 	const QString firstLine(QString::number(revId) + ".");
 	int idx = 0;
-	loopList(it, curAnn->lines) {
+	FOREACH_SL (it, curAnn->lines) {
 		if ((*it).stripWhiteSpace().startsWith(firstLine)) {
 			ft->setSelection(idx, 0, idx, (*it).length());
 			return;
@@ -334,8 +334,11 @@ bool FileContent::lookupAnnotation() {
 		curAnn = git->lookupAnnotation(annotateObj, st->fileName(), st->sha());
 
 		if (curAnn) {
-			annoLen = annotateLength(curAnn);
-			curAnnIt = curAnn->lines.constBegin();
+			if (!curAnn->lines.empty()) {
+				annoLen = annotateLength(curAnn);
+				curAnnIt = curAnn->lines.constBegin();
+			} else
+				curAnn = NULL;
 		} else {
 			dbp("ASSERT in lookupAnnotation: no annotation for %1", st->fileName());
 			clearAnnotate();
@@ -469,7 +472,7 @@ uint FileContent::processData(SCRef fileChunk) {
 	bool isHtmlHeader = (isHtmlSource && curLine == 1);
 	bool isHtmlFirstContentLine = false;
 
-	loopList(it, sl) {
+	FOREACH_SL (it, sl) {
 
 		if (isHtmlHeader) {
 			if ((*it).startsWith(HTML_FILE_START)) {
