@@ -103,35 +103,35 @@ void ListBoxFiles::insertFiles(const RevFile* files) {
 	if (files->names.empty())
 		return;
 
-	int prevPar = files->mergeParent[0];
-	for (uint i = 0; i < files->names.count(); ++i) {
+	bool isMergeParents = !files->mergeParent.isEmpty();
+	int prevPar = (isMergeParents ? files->mergeParent.first() : 1);
+	for (int i = 0; i < files->count(); ++i) {
 
-		QChar status(files->getStatus(i));
-		if (status == QGit::UNKNOWN)
+		if (files->statusCmp(i, RevFile::UNKNOWN))
 			continue;
 
 		QColor clr = Qt::black;
-		if (files->mergeParent[i] != prevPar) {
+		if (isMergeParents && files->mergeParent[i] != prevPar) {
 			prevPar = files->mergeParent[i];
 			new ListBoxFileItem(lb, "", clr);
 			new ListBoxFileItem(lb, "", clr);
 		}
-		QString extSt(files->getExtendedStatus(i));
+		QString extSt(files->extendedStatus(i));
 		if (extSt.isEmpty()) {
-			if (status == QGit::MODIFIED)
+			if (files->statusCmp(i, RevFile::MODIFIED))
 				; // common case
-			else if (status == QGit::NEW)
+			else if (files->statusCmp(i, RevFile::NEW))
 				clr = Qt::darkGreen;
-			else if (status == QGit::DELETED)
+			else if (files->statusCmp(i, RevFile::DELETED))
 				clr = Qt::red;
 		} else {
 			clr = Qt::darkBlue;
 			// in case of rename deleted file is not shown and...
-			if (status == QGit::DELETED)
+			if (files->statusCmp(i, RevFile::DELETED))
 				continue;
 
 			// ...new file is shown with extended info
-			if (status == QGit::NEW) {
+			if (files->statusCmp(i, RevFile::NEW)) {
 				new ListBoxFileItem(lb, extSt, clr);
 				continue;
 			}
