@@ -158,13 +158,13 @@ bool FileView::doUpdate(bool force) {
 
 		m()->statusBar()->message("Retrieving history of '" +
 		                          st.fileName() + "'...");
-		fh->reset(st.fileName());
+		fh->clear(st.fileName());
 		git->startFileHistory(fh);
 
 	} else if (histListView->update() || st.sha().isEmpty()) {
 
 		updateSpinBoxValue();
-		m()->statusBar()->message(git->getRevInfo(st.sha(), fh));
+		m()->statusBar()->message(git->getRevInfo(st.sha()));
 	}
 	if (!fileTab->toolButtonPin->isOn())
 		textEditFile->update();
@@ -327,7 +327,7 @@ void FileView::on_fileAvailable(bool b) {
 	updateEnabledButtons();
 
 	if (b) {
-		// code range is indipendent from annotation
+		// code range is independent from annotation
 		if (fileTab->toolButtonRangeFilter->isOn())
 			textEditFile->goToRangeStart();
 
