@@ -6,6 +6,7 @@
 	Copyright: See COPYING file that comes with this distribution
 
 */
+#include <stdlib.h>    // used by getenv()
 #include <sys/types.h> // used by chmod()
 #include <sys/stat.h>  // used by chmod()
 #include <qapplication.h>
@@ -17,32 +18,51 @@
 #include <qdir.h>
 #include <qeventloop.h>
 #include <qprocess.h>
-#include <qtextcodec.h>
 #include <qstylesheet.h>
 #include <qobjectlist.h>
-#include "mainimpl.h"
+#include "lanes.h"
+#include "myprocess.h"
 #include "annotate.h"
 #include "cache.h"
-#include "dataloader.h"
 #include "git.h"
 
 using namespace QGit;
 
+FileHistory::FileHistory() {
+
+	lns = new Lanes();
+	revs.setAutoDelete(true);
+	clear("");
+	revs.resize(QGit::MAX_DICT_SIZE);
+	rowData.setAutoDelete(true);
+}
+
+FileHistory::~FileHistory() {
+
+	delete lns;
+}
+
+void FileHistory::clear(SCRef name) {
+
+	revs.clear();
+	revOrder.clear();
+	firstFreeLane = 0;
+	lns->clear();
+	fileName = name;
+	rowData.clear();
+}
+
 Git::Git(QWidget* p) : QObject(p) {
 
 	EM_INIT(exGitStopped, "Stopping connection with git");
 
-	cacheNeedsUpdate = isMergeHead = false;
-	isStGIT = isGIT = loadingUnAppliedPatches = false;
+	fileCacheAccessed = cacheNeedsUpdate = isMergeHead = false;
+	isStGIT = isGIT = loadingUnAppliedPatches = isTextHighlighterFound = false;
 	errorReportingEnabled = true; // report errors if run() fails
 	curDomain = NULL;
 
-	revs.resize(MAX_DICT_SIZE);
 	revsFiles.resize(MAX_DICT_SIZE);
-	revs.setAutoDelete(true);
 	revsFiles.setAutoDelete(true);
-
-	cache = new Cache(this);
 }
 
 void Git::checkEnvironment() {
@@ -72,6 +92,34 @@ void Git::checkEnvironment() {
 		dbp("Found %1", version.section('\n', 0, 0));
 }
 
+void Git::userInfo(SList info) {
+/*
+  git looks for commit user information in following order:
+
+	- GIT_AUTHOR_NAME and GIT_AUTHOR_EMAIL environment variables
+	- repository config file
+	- global config file
+	- your name, hostname and domain
+*/
+	QString user(getenv("GIT_AUTHOR_NAME"));
+	QString email(getenv("GIT_AUTHOR_EMAIL"));
+
+	info.clear();
+	info << "Environment" << user << email;
+
+	errorReportingEnabled = false; // 'git repo-config' could fail, see docs
+
+	run("git repo-config user.name", &user);
+	run("git repo-config user.email", &email);
+	info << "Local config" << user << email;
+
+	run("git repo-config --global user.name", &user);
+	run("git repo-config --global user.email", &email);
+	info << "Global config" << user << email;
+
+	errorReportingEnabled = true;
+}
+
 void Git::setThrowOnStop(bool b) {
 
 	if (b)
@@ -96,9 +144,9 @@ bool Git::allProcessDeleted() {
 void Git::setTextCodec(QTextCodec* tc) {
 
 	QTextCodec::setCodecForCStrings(tc); // works also with tc == 0 (Latin1)
-	QString mimeName((tc != NULL) ? tc->mimeName() : "Latin1");
+	QString mimeName(tc ? tc->mimeName() : "Latin1");
 
-	// workaround Qt issue of mime name diffrent from
+	// workaround Qt issue of mime name different from
 	// standard http://www.iana.org/assignments/character-sets
 	if (mimeName == "Big5-HKSCS")
 		mimeName = "Big5";
@@ -141,79 +189,201 @@ const QString Git::getLocalDate(SCRef gitDate) {
 	return d.toString(Qt::LocalDate);
 }
 
-const QString Git::getRevInfo(SCRef sha, FileHistory* fh) {
+uint Git::checkRef(SCRef sha, uint mask) const {
 
-	const Rev* c = revLookup(sha, fh);
-	if (c == NULL)
+	QMap<QString, Reference>::const_iterator it(refsShaMap.find(sha));
+	return (it != refsShaMap.constEnd() ? (*it).type & mask : 0);
+}
+
+const QStringList Git::getRefName(SCRef sha, RefType type, QString* curBranch) const {
+
+	if (!checkRef(sha, type))
+		return QStringList();
+
+	const Reference& rf = refsShaMap[sha];
+
+	if (curBranch)
+		*curBranch = rf.currentBranch;
+
+	if (type == TAG)
+		return rf.tags;
+
+	else if (type == BRANCH)
+		return rf.branches;
+
+	else if (type == RMT_BRANCH)
+		return rf.remoteBranches;
+
+	else if (type == REF)
+		return rf.refs;
+
+	else if (type == APPLIED || type == UN_APPLIED)
+		return QStringList(rf.stgitPatch);
+
+	return QStringList();
+}
+
+const QStringList Git::getAllRefSha(uint mask) {
+
+	QStringList shas;
+	FOREACH (RefMap, it, refsShaMap)
+		if ((*it).type & mask)
+			shas.append(it.key());
+	return shas;
+}
+
+const QString Git::getRefSha(SCRef refName, RefType type, bool askGit) {
+
+	bool any = (type == ANY_REF);
+
+	FOREACH (RefMap, it, refsShaMap) {
+
+	        const Reference& rf = *it;
+
+		if ((any || type == TAG) && rf.tags.contains(refName))
+			return it.key();
+
+		else if ((any || type == BRANCH) && rf.branches.contains(refName))
+			return it.key();
+
+		else if ((any || type == RMT_BRANCH) && rf.remoteBranches.contains(refName))
+			return it.key();
+
+		else if ((any || type == REF) && rf.refs.contains(refName))
+			return it.key();
+
+		else if ((any || type == APPLIED || type == UN_APPLIED) && rf.stgitPatch == refName)
+			return it.key();
+	}
+	if (!askGit)
+		return "";
+
+	// if a ref was not found perhaps is an abbreviated form
+	QString runOutput;
+	errorReportingEnabled = false;
+	bool ok = run("git rev-parse " + refName, &runOutput);
+	errorReportingEnabled = true;
+	return (ok ? runOutput.stripWhiteSpace() : "");
+}
+
+void Git::appendNamesWithId(QStringList& names, SCRef sha, SCList data, bool onlyLoaded) {
+
+	const Rev* r = revLookup(sha);
+	if (onlyLoaded && !r)
+		return;
+
+	if (onlyLoaded) { // prepare for later sorting
+		SCRef cap = QString("%1 ").arg(r->orderIdx, 6);
+		FOREACH_SL (it, data)
+			names.append(cap + *it);
+	} else
+		names += data;
+}
+
+const QStringList Git::getAllRefNames(uint mask, bool onlyLoaded) {
+// returns reference names sorted by loading order if 'onlyLoaded' is set
+
+	QStringList names;
+	FOREACH (RefMap, it, refsShaMap) {
+
+		if (mask & TAG)
+			appendNamesWithId(names, it.key(), (*it).tags, onlyLoaded);
+
+		if (mask & BRANCH)
+			appendNamesWithId(names, it.key(), (*it).branches, onlyLoaded);
+
+		if (mask & RMT_BRANCH)
+			appendNamesWithId(names, it.key(), (*it).remoteBranches, onlyLoaded);
+
+		if (mask & REF)
+			appendNamesWithId(names, it.key(), (*it).refs, onlyLoaded);
+
+		if ((mask & (APPLIED | UN_APPLIED)) && !onlyLoaded)
+			names.append((*it).stgitPatch); // doesn't work with 'onlyLoaded'
+        }
+        if (onlyLoaded) {
+		names.sort();
+		QStringList::iterator itN(names.begin());
+		for ( ; itN != names.end(); ++itN) // strip 'idx'
+			(*itN) = (*itN).section(' ', -1, -1);
+	}
+	return names;
+}
+
+const QString Git::getRevInfo(SCRef sha) {
+
+	uint type = checkRef(sha);
+	if (type == 0)
 		return "";
 
 	QString refsInfo;
-	if (c->isBranch) {
-		if (c->isCurrentBranch)
-			refsInfo = "Head: " + c->branch;
-		else
-			refsInfo = "Branch: " + c->branch;
+	if (type & BRANCH) {
+		const QString cap(type & CUR_BRANCH ? "Head: " : "Branch: ");
+		refsInfo =  cap + getRefName(sha, BRANCH).join(" ");
 	}
-	if (c->isTag)
-		refsInfo.append("   Tag: " + c->tag);
-	if (c->isRef)
-		refsInfo.append("   Ref: " + c->ref);
-	if (c->isApplied || c->isUnApplied)
-		refsInfo.append("   Patch: " + getPatchName(sha));
+	if (type & RMT_BRANCH)
+		refsInfo.append("   Remote branch: " + getRefName(sha, RMT_BRANCH).join(" "));
 
-	refsInfo = refsInfo.stripWhiteSpace();
+	if (type & TAG)
+		refsInfo.append("   Tag: " + getRefName(sha, TAG).join(" "));
 
-	if (!refsInfo.isEmpty()) {
+	if (type & REF)
+		refsInfo.append("   Ref: " + getRefName(sha, REF).join(" "));
+
+	if (type & APPLIED)
+		refsInfo.append("   Patch: " + getRefName(sha, APPLIED).join(" "));
+
+	if (type & UN_APPLIED)
+		refsInfo.append("   Patch: " + getRefName(sha, UN_APPLIED).join(" "));
+
+	if (type & TAG) {
 		SCRef msg(getTagMsg(sha));
 		if (!msg.isEmpty())
 			refsInfo.append("  [" + msg + "]");
 	}
-	return refsInfo;
+	return refsInfo.stripWhiteSpace();
 }
 
-const QString Git::getRefSha(SCRef refName) {
+const QString Git::getTagMsg(SCRef sha) {
 
-	if (tags.contains(refName))
-		return tagsSHA[tags.findIndex(refName)];
-
-	if (heads.contains(refName))
-		return headsSHA[heads.findIndex(refName)];
-
-	if (refs.contains(refName))
-		return refsSHA[refs.findIndex(refName)];
-
-	// if a ref was not found perhaps is an abbreviated form
-	QString runOutput;
-	if (!run("git rev-parse " + refName, &runOutput))
+	if (!checkRef(sha, TAG)) {
+		dbs("ASSERT in Git::getTagMsg, tag not found");
 		return "";
+	}
+	Reference& rf = refsShaMap[sha];
 
-	return runOutput.stripWhiteSpace();
+	if (!rf.tagMsg.isEmpty())
+		return rf.tagMsg;
+
+	QRegExp pgp("-----BEGIN PGP SIGNATURE*END PGP SIGNATURE-----", true, true);
+
+	if (!rf.tagObj.isEmpty()) {
+		QString ro;
+		if (run("git cat-file tag " + rf.tagObj, &ro))
+			rf.tagMsg = ro.section("\n\n", 1).remove(pgp).stripWhiteSpace();
+	}
+	return rf.tagMsg;
 }
 
-bool Git::isPatchName(SCRef patchName) {
+bool Git::isPatchName(SCRef nm) {
 
-	return patchNames.values().contains(patchName);
+	if (!getRefSha(nm, UN_APPLIED, false).isEmpty())
+		return true;
+
+	return !getRefSha(nm, APPLIED, false).isEmpty();
 }
 
-bool Git::isTagName(SCRef tag) { return tags.contains(tag); }
-
-bool Git::isCommittingMerge() { return isMergeHead; }
-
-bool Git::isStGITStack() { return isStGIT; }
-
-bool Git::isUnapplied(SCRef sha) { return unAppliedSHA.contains(sha); }
-
 void Git::addExtraFileInfo(QString* rowName, SCRef sha, SCRef diffToSha, bool allMergeFiles) {
 
-	const RevFile* files;
-	if ((files = getFiles(sha, diffToSha, allMergeFiles)) == NULL)
+	const RevFile* files = getFiles(sha, diffToSha, allMergeFiles);
+	if (!files)
 		return;
 
-	int idx;
-	if ((idx = findFileIndex(*files, *rowName)) == -1)
+	int idx = findFileIndex(*files, *rowName);
+	if (idx == -1)
 		return;
 
-	QString extSt(files->getExtendedStatus(idx));
+	QString extSt(files->extendedStatus(idx));
 	if (extSt.isEmpty())
 		return;
 
@@ -228,13 +398,9 @@ void Git::removeExtraFileInfo(QString* rowName) {
 
 void Git::formatPatchFileHeader(QString* rowName, SCRef sha, SCRef diffToSha,
                                 bool combined, bool allMergeFiles) {
-
-	// return git patch format: diff --git a/<origFile> b/<destFile>
-	QString prefix((combined) ? "diff --combined " : "diff --git a/");
-
-	if (combined) { // TODO rename/copy still not supported in this case
-		*rowName = prefix + *rowName;
-		return;
+	if (combined) {
+		rowName->prepend("diff --combined ");
+		return; // TODO rename/copy still not supported in this case
 	}
 	// let's see if it's a rename/copy...
 	addExtraFileInfo(rowName, sha, diffToSha, allMergeFiles);
@@ -243,16 +409,16 @@ void Git::formatPatchFileHeader(QString* rowName, SCRef sha, SCRef diffToSha,
 
 		SCRef destFile(rowName->section(" --> ", 1, 1).section(" (", 0, 0));
 		SCRef origFile(rowName->section(" --> ", 0, 0));
-		*rowName = prefix + origFile + " b/" + destFile;
+		*rowName = "diff --git a/" + origFile + " b/" + destFile;
 	} else
-		*rowName = prefix + *rowName + " b/" + *rowName;
+		*rowName = "diff --git a/" + *rowName + " b/" + *rowName;
 }
 
 Annotate* Git::startAnnotate(FileHistory* fh, QObject* guiObj) { // non blocking
 
 	Annotate* ann = new Annotate(this, guiObj);
 	ann->start(fh); // non blocking call
-	return ann;
+	return ann; // caller will delete with Git::cancelAnnotate()
 }
 
 void Git::cancelAnnotate(Annotate* ann) {
@@ -263,18 +429,15 @@ void Git::cancelAnnotate(Annotate* ann) {
 
 void Git::annotateExited(Annotate* ann) {
 
-	const QString msg = QString("Annotated %1 files in %2 ms")
-	                   .arg(ann->count()).arg(ann->elapsed());
+	SCRef msg = QString("Annotated %1 files in %2 ms")
+	            .arg(ann->count()).arg(ann->elapsed());
 
 	emit annotateReady(ann, ann->file(), ann->isValid(), msg);
 }
 
 const FileAnnotation* Git::lookupAnnotation(Annotate* ann, SCRef fileName, SCRef sha) {
 
-	if (!ann)
-		return NULL;
-
-	return ann->lookupAnnotation(sha, fileName);
+	return (ann ? ann->lookupAnnotation(sha, fileName) : NULL);
 }
 
 void Git::cancelDataLoading(const FileHistory* fh) {
@@ -283,9 +446,9 @@ void Git::cancelDataLoading(const FileHistory* fh) {
 	emit cancelLoading(fh); // non blocking
 }
 
-const Rev* Git::revLookup(SCRef sha, const FileHistory* fh) {
+const Rev* Git::revLookup(SCRef sha, const FileHistory* fh) const {
 
-	const RevMap& r = (fh ? fh->revs : revs);
+	const RevMap& r = (fh ? fh->revs : revData.revs);
 	return r[sha];
 }
 
@@ -339,7 +502,7 @@ int Git::findFileIndex(const RevFile& rf, SCRef name) {
 	SCRef dr = name.left(idx);
 	SCRef nm = name.mid(idx);
 
-	for (uint i = 0; i < rf.names.count(); ++i) {
+	for (uint i = 0, cnt = rf.count(); i < cnt; ++i) {
 		if (fileNamesVec[rf.names[i]] == nm && dirNamesVec[rf.dirs[i]] == dr)
 			return i;
 	}
@@ -354,7 +517,7 @@ const QString Git::getLaneParent(SCRef fromSHA, uint laneNum) {
 
 	for (int idx = rs->orderIdx - 1; idx >= 0; idx--) {
 
-		const Rev* r = revLookup(revOrder[idx]);
+		const Rev* r = revLookup(revData.revOrder[idx]);
 		if (laneNum >= r->lanes.count())
 			return "";
 
@@ -382,13 +545,13 @@ const QStringList Git::getChilds(SCRef parent) {
 		return childs;
 
 	for (uint i = 0; i < r->childs.count(); i++)
-		childs.append(revOrder[r->childs[i]]);
+		childs.append(revData.revOrder[r->childs[i]]);
 
 	// reorder childs by loading order
 	QStringList::iterator itC(childs.begin());
 	for ( ; itC != childs.end(); ++itC) {
 		const Rev* r = revLookup(*itC);
-		(*itC).prepend(QString("%1 ").arg(r->orderIdx, 5));
+		(*itC).prepend(QString("%1 ").arg(r->orderIdx, 6));
 	}
 	childs.sort();
 	for (itC = childs.begin(); itC != childs.end(); ++itC)
@@ -403,41 +566,6 @@ const QString Git::getShortLog(SCRef sha) {
 	return (r ? r->shortLog() : "");
 }
 
-const QStringList Git::getTagNames(bool onlyLoaded) {
-
-	return (onlyLoaded) ? loadedTagNames : tags;
-}
-
-const QStringList Git::getBranchNames() {
-
-	return loadedBranchNames;
-}
-
-const QString Git::getTagMsg(SCRef sha) {
-
-	Rev* c = const_cast<Rev*>(revLookup(sha));
-	if (!c)
-		return "";
-
-	if (!c->tagMsg.isEmpty())
-		return c->tagMsg;
-
-	QRegExp pgp = QRegExp("-----BEGIN PGP SIGNATURE*END PGP SIGNATURE-----", true, true);
-
-	if (tagsObj.contains(sha)) {
-
-		QString runOutput;
-		if (run("git cat-file tag " + tagsObj[sha], &runOutput)) {
-
-			c->tagMsg = runOutput.section("\n\n", 1);
-			if (!c->tagMsg.isEmpty())
-				c->tagMsg = c->tagMsg.remove(pgp).stripWhiteSpace();
-		}
-		tagsObj.remove(sha);
-	}
-	return c->tagMsg;
-}
-
 MyProcess* Git::getDiff(SCRef sha, QObject* receiver, SCRef diffToSha, bool combined) {
 
 	if (sha.isEmpty())
@@ -456,13 +584,12 @@ MyProcess* Git::getDiff(SCRef sha, QObject* receiver, SCRef diffToSha, bool comb
 
 const QString Git::getFileSha(SCRef file, SCRef revSha) {
 
-	const QString sha(revSha == ZERO_SHA ? "HEAD" : revSha);
+	const QString sha(revSha == ZERO_SHA ? "HEAD" : revSha); // FIXME ZERO_SHA != HEAD
 	QString runCmd("git ls-tree -r " + sha + " " + quote(file)), runOutput;
-	bool ok = run(runCmd, &runOutput);
-	if (!ok || runOutput.isEmpty()) // deleted file case
+	if (!run(runCmd, &runOutput))
 		return "";
 
-	return runOutput.mid(12, 40);
+	return runOutput.mid(12, 40); // could be empty, deleted file case
 }
 
 MyProcess* Git::getFile(SCRef file, SCRef revSha, QObject* receiver, QString* result) {
@@ -477,7 +604,7 @@ MyProcess* Git::getFile(SCRef file, SCRef revSha, QObject* receiver, QString* re
 		else
 			runCmd = "git diff-tree HEAD HEAD"; // fake an empty file reading
 	}
-	if (receiver == NULL) {
+	if (!receiver) {
 		run(runCmd, result);
 		return NULL; // in case of sync call we ignore run() return value
 	}
@@ -497,7 +624,7 @@ MyProcess* Git::getHighlightedFile(SCRef file, SCRef sha, QObject* receiver, QSt
 
 	QString runCmd("source-highlight --failsafe -f html -i " + quote(inputFile));
 
-	if (receiver == NULL) {
+	if (!receiver) {
 		run(runCmd, result);
 		on_getHighlightedFile_eof();
 		return NULL; // in case of sync call we ignore run() return value
@@ -512,7 +639,7 @@ void Git::on_getHighlightedFile_eof() {
 
 	QDir dir(workDir);
 	const QStringList sl(dir.entryList("qgit_hlght_input*"));
-	loopList(it, sl)
+	FOREACH_SL (it, sl)
 		dir.remove(*it);
 }
 
@@ -529,14 +656,14 @@ bool Git::getTree(SCRef treeSha, SList names, SList shas,
 	QStringList newFiles, unfiles, delFiles, dummy;
 	if (isWorkingDir) { // retrieve unknown and deleted files under treePath
 
-		getWorkDirFiles(UNKNOWN, unfiles, dummy);
-		loopList(it, unfiles) { // don't add unknown files under other directories
+		getWorkDirFiles(unfiles, dummy, RevFile::UNKNOWN);
+		FOREACH_SL (it, unfiles) { // don't add unknown files under other directories
 			QFileInfo f(*it);
 			SCRef d(f.dirPath(false));
 			if (d == treePath || (treePath.isEmpty() && d == "."))
 				newFiles.append(f.fileName());
 		}
-		getWorkDirFiles(DELETED, delFiles, dummy);
+		getWorkDirFiles(delFiles, dummy, RevFile::DELETED);
 	}
 	// if needed fake a working directory tree starting from HEAD tree
 	const QString tree(treeSha == ZERO_SHA ? "HEAD" : treeSha);
@@ -545,14 +672,14 @@ bool Git::getTree(SCRef treeSha, SList names, SList shas,
 		return false;
 
 	const QStringList sl(QStringList::split('\n', runOutput));
-	loopList(it, sl) {
+	FOREACH_SL (it, sl) {
 		// insert in order any good unknown file to the list,
 		// newFiles must be already sorted
 		SCRef fn((*it).section('\t', 1, 1));
 		while (!newFiles.empty() && newFiles.first() < fn) {
 			names.append(newFiles.first());
 			shas.append("");
-			types.append(NEW);
+			types.append("?"); // FIXME test
 			newFiles.pop_front();
 		}
 		// append any not deleted file
@@ -566,13 +693,13 @@ bool Git::getTree(SCRef treeSha, SList names, SList shas,
 	while (!newFiles.empty()) { // append any remaining unknown file
 		names.append(newFiles.first());
 		shas.append("");
-		types.append(NEW);
+		types.append("?"); // FIXME test
 		newFiles.pop_front();
 	}
 	return true;
 }
 
-void Git::getWorkDirFiles(const QChar& status, SList files, SList dirs) {
+void Git::getWorkDirFiles(SList files, SList dirs, RevFile::StatusFlag status) {
 
 	files.clear();
 	dirs.clear();
@@ -580,14 +707,13 @@ void Git::getWorkDirFiles(const QChar& status, SList files, SList dirs) {
 	if (!f)
 		return;
 
-	for (uint i = 0; i < f->names.count(); i++) {
+	for (int i = 0; i < f->count(); i++) {
 
 		if (f->statusCmp(i, status)) {
 
 			SCRef fp(filePath(*f, i));
 			files.append(fp);
-
-			for (int j = 0; j < fp.contains('/'); j++) {
+			for (int j = 0, cnt = fp.contains('/'); j < cnt; j++) {
 
 				SCRef dir(fp.section('/', 0, j));
 				if (dirs.findIndex(dir) == -1)
@@ -597,24 +723,29 @@ void Git::getWorkDirFiles(const QChar& status, SList files, SList dirs) {
 	}
 }
 
+bool Git::isNothingToCommit() {
+
+	const RevFile* rf = revsFiles[ZERO_SHA];
+	return (!rf || ((uint)rf->count() == _wd.otherFiles.count()));
+}
+
 bool Git::isTreeModified(SCRef sha) {
 
 	const RevFile* f = getFiles(sha);
-	if (f) {
-		for (uint i = 0; i < f->names.count(); ++i)
-			if (!f->statusCmp(i, MODIFIED))
-				return true;
-	}
+	if (!f)
+		return true; // no files info, stay on the safe side
+
+	for (int i = 0; i < f->count(); ++i)
+		if (!f->statusCmp(i, RevFile::MODIFIED))
+			return true;
+
 	return false;
 }
 
 bool Git::isParentOf(SCRef par, SCRef child) {
 
 	const Rev* c = revLookup(child);
-	if (!c || c->parentsCount() != 1) // we don't handle merges
-		return false;
-
-	return (c->parent(0) == par);
+	return (c && c->parentsCount() == 1 && c->parent(0) == par); // no merges
 }
 
 bool Git::isSameFiles(SCRef tree1Sha, SCRef tree2Sha) {
@@ -644,17 +775,15 @@ const QStringList Git::getDescendantBranches(SCRef sha) {
 	if (!r || (r->descBrnMaster == -1))
 		return tl;
 
-	const QValueVector<int>& nr = revLookup(revOrder[r->descBrnMaster])->descBranches;
+	const QValueVector<int>& nr = revLookup(revData.revOrder[r->descBrnMaster])->descBranches;
 
 	for (uint i = 0; i < nr.count(); i++) {
 
-		SCRef sha = revOrder[nr[i]];
-		int idx = headsSHA.findIndex(sha);
-		if (idx != -1) {
-			SCRef branchName = heads[idx];
-			const QString t = branchName + " (" + sha + ")";
-			tl.append(t);
-		}
+		SCRef sha = revData.revOrder[nr[i]];
+		SCRef cap = " (" + sha + ") ";
+		RefMap::const_iterator it(refsShaMap.find(sha));
+		if (it != refsShaMap.constEnd())
+			tl.append((*it).branches.join(cap).append(cap));
 	}
 	return tl;
 }
@@ -670,24 +799,30 @@ const QStringList Git::getNearTags(bool goDown, SCRef sha) {
 	if (nearRefsMaster == -1)
 		return tl;
 
-	const QValueVector<int>& nr = goDown ? revLookup(revOrder[nearRefsMaster])->descRefs :
-	                                       revLookup(revOrder[nearRefsMaster])->ancRefs;
+	const QValueVector<int>& nr = goDown ? revLookup(revData.revOrder[nearRefsMaster])->descRefs :
+	                                       revLookup(revData.revOrder[nearRefsMaster])->ancRefs;
 
 	for (uint i = 0; i < nr.count(); i++) {
 
-		SCRef sha = revOrder[nr[i]];
-		int idx = tagsSHA.findIndex(sha);
-		if (idx != -1)
-			tl.append(tags[idx] + " (" + sha + ")");
+		SCRef sha = revData.revOrder[nr[i]];
+		SCRef cap = " (" + sha + ")";
+		RefMap::const_iterator it(refsShaMap.find(sha));
+		if (it != refsShaMap.constEnd())
+			tl.append((*it).tags.join(cap).append(cap));
 	}
 	return tl;
 }
 
 const QString Git::getDefCommitMsg() {
 
-	SCRef sha(appliedSHA.empty() ? ZERO_SHA : appliedSHA.last());
+	QString sha(ZERO_SHA);
+	if (isStGIT && !getAllRefSha(APPLIED).isEmpty()) {
+		QString top;
+		if (run("stg top", &top))
+			sha = getRefSha(top.stripWhiteSpace(), APPLIED, false);
+	}
 	const Rev* c = revLookup(sha);
-	if (c == NULL) {
+	if (!c) {
 		dbp("ASSERT: getDefCommitMsg sha <%1> not found", sha);
 		return "";
 	}
@@ -723,14 +858,8 @@ const QString Git::getDesc(SCRef sha, QRegExp& shortLogRE, QRegExp& longLogRE) {
 		return "";
 
 	const Rev* c = revLookup(sha);
-	if (c == NULL) {
-		dbp("ASSERT in Git::getDesc: sha <%1> not found", sha);
-		return "";
-	}
-
-	// set temporary Latin-1 to avoid using QString::fromLatin1() everywhere
-	QTextCodec* tc = QTextCodec::codecForCStrings();
-	QTextCodec::setCodecForCStrings(0);
+	if (!c)            // sha of a not loaded revision, as
+		return ""; // example asked from file history
 
 	QString text;
 	if (c->isDiffCache)
@@ -738,7 +867,7 @@ const QString Git::getDesc(SCRef sha, QRegExp& shortLogRE, QRegExp& longLogRE) {
 	else {
 		text = QString("Author: " + c->author() + "\nDate:   ");
 		text.append(getLocalDate(c->authorDate()));
-		if (!c->isUnApplied) {
+		if (!c->isUnApplied && !c->isApplied) {
 			text.append("\nParent: ").append(c->parents().join("\nParent: "));
 
 			QStringList sl = getChilds(sha);
@@ -760,54 +889,71 @@ const QString Git::getDesc(SCRef sha, QRegExp& shortLogRE, QRegExp& longLogRE) {
 		text.append("\n\n    " + colorMatch(c->shortLog(), shortLogRE) +
 		            '\n' + colorMatch(c->longLog(), longLogRE));
 	}
+	// workaround a Qt issue
+	QTextCodec* tc = QTextCodec::codecForCStrings();
+	QTextCodec::setCodecForCStrings(0);
 	text = QStyleSheet::convertFromPlainText(text); // this puppy needs Latin-1
+	QTextCodec::setCodecForCStrings(tc);
 
-	// hightlight SHA's
+	// highlight SHA's
+	//
+	// search for abbreviated sha too. To filter out debug backtraces sometimes
+	// added to commit logs, we avoid to call git rev-parse for a possible abbreviated
+	// sha if there isn't a leading trailing space or an open parenthesis and,
+	// in that case, before the space must not be a ':' character.
+	// It's an ugly heuristic, but seems to work in most cases.
+	QRegExp reSHA("..[0-9a-f]{21,40}|[^:][\\s(][0-9a-f]{6,20}", false);
+	reSHA.setMinimal(false);
 	int pos = 0;
-	QRegExp reSHA("[0-9a-f]{40}", false);
-	reSHA.setMinimal(true);
 	while ((pos = text.find(reSHA, pos)) != -1) {
 
-		SCRef sha(reSHA.cap(0));
-		const Rev* r = revLookup(sha);
-		QString slog(r ? r->shortLog() : sha);
-		if (slog.isEmpty()) // very rare but possible
-			slog = sha;
-		if (slog.length() > 60)
-			slog = slog.left(57).stripWhiteSpace().append("...");
+		SCRef ref = reSHA.cap(0).mid(2);
+		const Rev* r = (ref.length() == 40 ? revLookup(ref) : revLookup(getRefSha(ref)));
+		if (r) {
+			QString slog(r->shortLog());
+			if (slog.isEmpty()) // very rare but possible
+				slog = r->sha();
+			if (slog.length() > 60)
+				slog = slog.left(57).stripWhiteSpace().append("...");
 
-		slog = QStyleSheet::escape(slog);
-		const QString link("<a href=\"" + sha + "\">" + slog + "</a>");
-		text.replace(pos, sha.length(), link);
-		pos += link.length();
+			slog = QStyleSheet::escape(slog);
+			const QString link("<a href=\"" + r->sha() + "\">" + slog + "</a>");
+			text.replace(pos + 2, ref.length(), link);
+			pos += link.length();
+		} else
+			pos += reSHA.cap(0).length();
 	}
 	text.replace("$_1", "<"); // see colorMatch()
 	text.replace("$_2", ">");
-
-	QTextCodec::setCodecForCStrings(tc); // restore codec
 	return text;
 }
 
+const RevFile* Git::insertNewFiles(SCRef sha, SCRef data) {
+
+	RevFile* rf = new RevFile();
+	parseDiffFormat(*rf, data);
+	revsFiles.insert(sha, rf);
+	return rf;
+}
+
 const RevFile* Git::getAllMergeFiles(const Rev* r) {
 
 	SCRef mySha(ALL_MERGE_FILES + r->sha());
 	RevFile* rf = revsFiles[mySha];
-	if (rf == NULL) {
-		QString runCmd("git diff-tree -r -m -C " + r->sha()), runOutput;
-		if (!run(runCmd, &runOutput))
-			return NULL;
+	if (rf)
+		return rf;
 
-		revsFiles.insert(mySha, new RevFile());
-		rf = revsFiles[mySha];
-		parseDiffFormat(*rf, runOutput);
-	}
-	return rf;
+	QString runCmd("git diff-tree -r -m -C " + r->sha()), runOutput;
+	if (!run(runCmd, &runOutput))
+		return NULL;
+
+	return insertNewFiles(mySha, runOutput);
 }
 
 const RevFile* Git::getFiles(SCRef sha, SCRef diffToSha, bool allFiles, SCRef path) {
 
 	const Rev* r = revLookup(sha);
-	if (r == NULL)
+	if (!r)
 		return NULL;
 
 	if (r->parentsCount() == 0) // skip initial rev
@@ -829,31 +975,26 @@ const RevFile* Git::getFiles(SCRef sha, SCRef diffToSha, bool allFiles, SCRef pa
 
 		// we insert a dummy revision file object. It will be
 		// overwritten at each request but we don't care.
-		revsFiles.insert(CUSTOM_SHA, new RevFile());
-		RevFile* rf = revsFiles[CUSTOM_SHA];
-		parseDiffFormat(*rf, runOutput);
+		return insertNewFiles(CUSTOM_SHA, runOutput);
+	}
+	RevFile* rf = revsFiles[sha];
+	if (rf)
+		return rf; // ZERO_SHA search arrives here
+
+	if (sha == ZERO_SHA) {
+		dbs("ASSERT in Git::getFiles, ZERO_SHA not found");
+		return NULL;
+	}
+	QString runCmd("git diff-tree -r -c -C " + sha), runOutput;
+	if (!run(runCmd, &runOutput))
+		return NULL;
+
+	rf = revsFiles[sha];
+	if (rf) // has been created in the mean time?
 		return rf;
-	}
-	RevFile* rf = revsFiles[sha]; // ZERO_SHA search arrives here
-	if (rf == NULL) {
 
-		if (sha == ZERO_SHA) {
-			dbs("ASSERT in Git::getFiles, ZERO_SHA not found");
-			return NULL;
-		}
-		QString runCmd("git diff-tree -r -c -C " + sha), runOutput;
-		if (!run(runCmd, &runOutput))
-			return false;
-
-		if (!revsFiles.find(sha)) { // has been created in the mean time?
-			revsFiles.insert(sha, new RevFile());
-			rf = revsFiles[sha];
-			parseDiffFormat(*rf, runOutput);
-			cacheNeedsUpdate = true;
-		} else
-			return revsFiles[sha];
-	}
-	return rf;
+	cacheNeedsUpdate = true;
+	return insertNewFiles(sha, runOutput);
 }
 
 void Git::startFileHistory(FileHistory* fh) {
@@ -865,15 +1006,15 @@ void Git::getFileFilter(SCRef path, QMap<QString, bool>& shaMap) {
 
 	shaMap.clear();
 	QRegExp rx(path, false, true); // not case sensitive and with wildcard
-	loop(StrVect, itr, revOrder) {
-		RevFile* rf = revsFiles[*itr];
-		if (rf == NULL)
+	FOREACH (StrVect, it, revData.revOrder) {
+		RevFile* rf = revsFiles[*it];
+		if (!rf)
 			continue;
 
 		// case insensitive, wildcard search
-		for (uint i = 0; i < rf->names.count(); ++i)
+		for (int i = 0; i < rf->count(); ++i)
 			if (rx.search(filePath(*rf, i)) != -1) {
-				shaMap.insert(*itr, true);
+				shaMap.insert(*it, true);
 				break;
 			}
 	}
@@ -883,7 +1024,7 @@ bool Git::getPatchFilter(SCRef exp, bool isRegExp, QMap<QString, bool>& shaMap)
 
 	shaMap.clear();
 	QString buf;
-	loop(StrVect, it, revOrder)
+	FOREACH (StrVect, it, revData.revOrder)
 		if (*it != ZERO_SHA)
 			buf.append(*it).append('\n');
 
@@ -895,12 +1036,12 @@ bool Git::getPatchFilter(SCRef exp, bool isRegExp, QMap<QString, bool>& shaMap)
 		runCmd.append("--pickaxe-regex ");
 
 	runCmd.append(quote("-S" + exp));
-	if (!run(runCmd, &runOutput, NULL, buf))
+	if (!run(runCmd, &runOutput, NULL, buf)) // could be slow
 		return false;
 
 	const QStringList sl(QStringList::split('\n', runOutput));
-	loopList(it2, sl)
-		shaMap.insert(*it2, true);
+	FOREACH_SL (it, sl)
+		shaMap.insert(*it, true);
 
 	return true;
 }
@@ -955,7 +1096,7 @@ bool Git::formatPatch(SCList shaList, SCRef dirPath, SCRef remoteDir) {
 
 	int n = shaList.count();
 	bool ret = false;
-	loopList(it, shaList) { // shaList is ordered by newest to oldest
+	FOREACH_SL (it, shaList) { // shaList is ordered by newest to oldest
 		const QString cmd(runCmd + QString::number(n) + " " +
 		                  *it + QString::fromLatin1("^..") + *it);
 		n--;
@@ -979,8 +1120,7 @@ bool Git::updateIndex(SCList selFiles) {
 
 bool Git::commitFiles(SCList selFiles, SCRef msg) {
 /*
-	Part of this code is taken from Fredrik Kuivinen "Gct"
-	tool. I have just translated from Python to C++
+	Part of this code is taken from Fredrik Kuivinen "Gct" tool
 */
 	const QString msgFile(gitDir + "/qgit_cmt_msg");
 	if (!writeToFile(msgFile, msg)) // early skip
@@ -1009,9 +1149,9 @@ bool Git::commitFiles(SCList selFiles, SCRef msg) {
 	// should avoid going through 'git commit'
 	QStringList selNotDelFiles;
 	const RevFile* files = getFiles(ZERO_SHA); // files != NULL
-	loopList(it, selFiles) {
+	FOREACH_SL (it, selFiles) {
 		int idx = findFileIndex(*files, *it);
-		if (!files->statusCmp(idx, DELETED))
+		if (!files->statusCmp(idx, RevFile::DELETED))
 			selNotDelFiles.append(*it);
 	}
 	// test if we need a git read-tree to temporary
@@ -1058,12 +1198,12 @@ const QStringList Git::getOtherFiles(SCList selFiles, bool onlyInIndex) {
 
 	const RevFile* files = getFiles(ZERO_SHA); // files != NULL
 	QStringList notSelFiles;
-	for (uint i = 0; i < files->names.count(); ++i) {
+	for (int i = 0; i < files->count(); ++i) {
 		SCRef fp = filePath(*files, i);
 		if (selFiles.find(fp) == selFiles.constEnd()) { // not selected...
 			if (!onlyInIndex)
 				notSelFiles.append(fp);
-			else if (files->isInIndex(i))
+			else if (files->statusCmp(i, RevFile::IN_INDEX))
 				notSelFiles.append(fp);
 		}
 	}
@@ -1073,14 +1213,14 @@ const QStringList Git::getOtherFiles(SCList selFiles, bool onlyInIndex) {
 void Git::removeFiles(SCList selFiles, SCRef workDir, SCRef ext) {
 
 	QDir d(workDir);
-	loopList(it, selFiles)
+	FOREACH_SL (it, selFiles)
 		d.rename(*it, *it + ext);
 }
 
 void Git::restoreFiles(SCList selFiles, SCRef workDir, SCRef ext) {
 
 	QDir d(workDir);
-	loopList(it, selFiles)
+	FOREACH_SL (it, selFiles)
 		d.rename(*it + ext, *it); // overwrites any existent file
 }
 
@@ -1088,9 +1228,9 @@ void Git::removeDeleted(SCList selFiles) {
 
 	QDir dir(workDir);
 	const RevFile* files = getFiles(ZERO_SHA); // files != NULL
-	loopList(it, selFiles) {
+	FOREACH_SL (it, selFiles) {
 		int idx = findFileIndex(*files, *it);
-		if (files->statusCmp(idx, DELETED))
+		if (files->statusCmp(idx, RevFile::DELETED))
 			dir.remove(*it);
 	}
 }
@@ -1106,8 +1246,8 @@ bool Git::stgCommit(SCList selFiles, SCRef msg, SCRef patchName, bool fold) {
 	const QString extS(".qgit_removed_selected");
 
 	// we have selected modified files in selFiles, we still need
-	// to know the not selected but modified files and, among
-	// theese the cached ones to proper restore state at the end.
+	// to know the modified but not selected files and, among
+	// these the cached ones to properly restore state at the end.
 	const QStringList notSelFiles = getOtherFiles(selFiles, !optOnlyInIndex);
 	const QStringList notSelInIndexFiles = getOtherFiles(selFiles, optOnlyInIndex);
 
@@ -1157,12 +1297,12 @@ error:
 
 exit:
 	// it is safe to call restore() also if back-up files don't
-	// exsist, so we can 'goto exit' from anywhere.
+	// exist, so we can 'goto exit' from anywhere.
 	restoreFiles(notSelFiles, workDir, extNS);
 	updateIndex(notSelInIndexFiles);
 	QDir dir(workDir);
 	dir.remove(patchFile);
-	loopList(it, selFiles)
+	FOREACH_SL (it, selFiles)
 		dir.remove(*it + extS); // remove temporary backup rollback files
 	return retval;
 }
@@ -1177,60 +1317,37 @@ bool Git::makeTag(SCRef sha, SCRef tagName, SCRef msg) {
 
 bool Git::deleteTag(SCRef sha) {
 
-	const Rev* r = revLookup(sha);
-	if (!r || !r->isTag)
-		return false;
+	const QStringList tags(getRefName(sha, TAG));
+	if (!tags.empty())
+		return run("git tag -d " + tags.first()); // only one
 
-	// r->tag could contain more then one tag, separated by '\n'
-	return run("git tag -d " + r->tag.section('\n', 0, 0));
+	return false;
 }
 
 bool Git::stgPush(SCRef sha) {
 
-	bool ret = run("stg push " + quote(patchNames[sha]));
-	if (!ret)
-		run("stg push --undo");
-
-	return ret;
+	const QStringList patch(getRefName(sha, UN_APPLIED));
+	if (patch.count() != 1) {
+		dbp("ASSERT in Git::stgPush, found %1 patches instead of 1", patch.count());
+		return false;
+	}
+	return run("stg push " + quote(patch.first()));
 }
 
 bool Git::stgPop(SCRef sha) {
 
-	if (sha == appliedSHA.last()) // top of the stack
-		return run("stg pop");
-
-	if (sha == appliedSHA.first())
-		return run("stg pop --all");
-
-	if (!run("stg pop " + quote(patchNames[sha])))
+	QString top;
+	if (!run("stg top", &top))
 		return false;
 
-	return run("stg pop");
-}
-
-bool Git::writeToFile(SCRef fileName, SCRef data) {
-
-	QFile file(QFile::encodeName(fileName));
-	if (!file.open(IO_WriteOnly)) {
-		dbp("ERROR: unable to write file %1", fileName);
+	const QStringList patch(getRefName(sha, APPLIED));
+	if (patch.count() != 1) {
+		dbp("ASSERT in Git::stgPop, found %1 patches instead of 1", patch.count());
 		return false;
 	}
-	QTextStream stream(&file);
-	stream << data;
-	file.close();
-	return true;
-}
+	if (patch.first() != top.stripWhiteSpace())
+		if (!run("stg pop " + quote(patch)))
+			return false;
 
-bool Git::readFromFile(SCRef fileName, QString& data) {
-
-	data = "";
-	QFile file(QFile::encodeName(fileName));
-	if (!file.open(IO_ReadOnly)) {
-		dbp("ERROR: unable to read file %1", fileName);
-		return false;
-	}
-	QTextStream stream(&file);
-	data = stream.read();
-	file.close();
-	return true;
+	return run("stg pop"); // finally remove selected one
 }
