@@ -7,45 +7,60 @@
 #ifndef GIT_H
 #define GIT_H
 
+#include "qobject.h"
 #include "exceptionmanager.h"
 #include "common.h"
-#include "lanes.h"
-#include "myprocess.h"
 
 class QRegExp;
 class QTextCodec;
-class Cache;
 class Annotate;
-class Domain;
+class Cache;
 class DataLoader;
+class Domain;
+class Lanes;
 class MyProcess;
 
 class FileHistory {
 public:
-	FileHistory() {
-		revs.setAutoDelete(true);
-		reset("");
-		revs.resize(QGit::MAX_DICT_SIZE);
-	}
-	void reset(SCRef name) {
-		revs.clear();
-		revOrder.clear();
-		firstFreeLane = 0;
-		lns.clear();
-		fileName = name;
-	}
+	FileHistory();
+	~FileHistory();
+	void clear(SCRef name);
 	QString fileName;
 	RevMap revs;
-	Lanes lns;
+	Lanes* lns;
 	StrVect revOrder;
 	uint firstFreeLane;
+	QPtrList<QByteArray> rowData;
 };
 
 class Git : public QObject {
 Q_OBJECT
 public:
 	explicit Git(QWidget* parent);
+
+	enum BoolOption { // used as self-documenting boolean parameters
+		optFalse,
+		optSaveCache,
+		optGoDown,
+		optOnlyLoaded,
+		optDragDrop,
+		optFold,
+		optOnlyInIndex
+	};
+
+	enum RefType {
+		TAG        = 1,
+		BRANCH     = 2,
+		RMT_BRANCH = 4,
+		CUR_BRANCH = 8,
+		REF        = 16,
+		APPLIED    = 32,
+		UN_APPLIED = 64,
+		ANY_REF    = 127
+	};
+
 	void checkEnvironment();
+	void userInfo(SList info);
 	const QString getBaseDir(bool* c, SCRef wd, bool* ok = NULL, QString* gd = NULL);
 	bool init(SCRef workDir, bool askForRange, QStringList* filterList, bool* quit);
 	bool stop(bool saveCache);
@@ -58,16 +73,14 @@ public:
 	void startFileHistory(FileHistory* fh);
 	void cancelDataLoading(const FileHistory* fh);
 	void cancelProcess(MyProcess* p);
-	bool isCommittingMerge(); // check for existence of MERGE_HEAD
-	bool isStGITStack();
-	bool isTagName(SCRef tag);
-	bool isPatchName(SCRef patchName);
-	bool isUnapplied(SCRef sha);
+	bool isCommittingMerge() const { return isMergeHead; }
+	bool isStGITStack() const { return isStGIT; }
+	bool isPatchName(SCRef nm);
 	bool isSameFiles(SCRef tree1Sha, SCRef tree2Sha);
-	bool isNothingToCommit() const { return nothingToCommit; }
-	bool isUnknownFiles() const { return unknownFiles; }
+	bool isNothingToCommit();
+	bool isUnknownFiles() const { return (_wd.otherFiles.count() > 0); }
 	bool isTextHighlighter() const { return isTextHighlighterFound; }
-	const Rev* revLookup(SCRef sha, const FileHistory* fh = NULL);
+	bool isMainHistory(const FileHistory* fh) { return (fh == &revData); }
 	MyProcess* getDiff(SCRef sha, QObject* receiver, SCRef diffToSha, bool combined);
 	MyProcess* getFile(SCRef file, SCRef revSha, QObject* receiver, QString* runOutput);
 	MyProcess* getHighlightedFile(SCRef file, SCRef revSha, QObject* rcv, QString* ro);
@@ -76,7 +89,7 @@ public:
 	bool getPatchFilter(SCRef exp, bool isRegExp, QMap<QString, bool>& shaMap);
 	const RevFile* getFiles(SCRef sha, SCRef sha2 = "", bool all = false, SCRef path = "");
 	bool getTree(SCRef ts, SList nm, SList sha, SList type, bool wd, SCRef treePath);
-	const QString getLocalDate(SCRef gitDate);
+	static const QString getLocalDate(SCRef gitDate);
 	const QString getDesc(SCRef sha, QRegExp& shortLogRE, QRegExp& longLogRE);
 	const QString getDefCommitMsg();
 	const QString getLaneParent(SCRef fromSHA, uint laneNum);
@@ -84,12 +97,15 @@ public:
 	const QStringList getNearTags(bool goDown, SCRef sha);
 	const QStringList getDescendantBranches(SCRef sha);
 	const QString getShortLog(SCRef sha);
-	const QStringList getTagNames(bool onlyLoaded = false);
-	const QStringList getBranchNames();
 	const QString getTagMsg(SCRef sha);
-	const QString getRefSha(SCRef refName);
-	const QString getRevInfo(SCRef sha, FileHistory* fh);
-	void getWorkDirFiles(const QChar& status, SList files, SList dirs);
+	const Rev* revLookup(SCRef sha, const FileHistory* fh = NULL) const;
+	uint checkRef(SCRef sha, uint mask = ANY_REF) const;
+	const QString getRevInfo(SCRef sha);
+	const QString getRefSha(SCRef refName, RefType type = ANY_REF, bool askGit = true);
+	const QStringList getRefName(SCRef sha, RefType type, QString* curBranch = NULL) const;
+	const QStringList getAllRefNames(uint mask, bool onlyLoaded);
+	const QStringList getAllRefSha(uint mask);
+	void getWorkDirFiles(SList files, SList dirs, RevFile::StatusFlag status);
 	QTextCodec* getTextCodec(bool* isGitArchive);
 	bool formatPatch(SCList shaList, SCRef dirPath, SCRef remoteDir = "");
 	bool updateIndex(SCList selFiles);
@@ -101,14 +117,11 @@ public:
 	bool stgCommit(SCList selFiles, SCRef msg, SCRef patchName, bool fold);
 	bool stgPush(SCRef sha);
 	bool stgPop(SCRef sha);
-	bool writeToFile(SCRef fileName, SCRef data);
-	bool readFromFile(SCRef fileName, QString& data);
 	void setTextCodec(QTextCodec* tc);
 	void addExtraFileInfo(QString* rowName, SCRef sha, SCRef diffToSha, bool allMergeFiles);
 	void removeExtraFileInfo(QString* rowName);
 	void formatPatchFileHeader(QString* rowName, SCRef sha, SCRef dts, bool cmb, bool all);
 	int findFileIndex(const RevFile& rf, SCRef name);
-	const QString getPatchName(SCRef sha) const { return patchNames[sha];	}
 	const QString filePath(const RevFile& rf, uint i) const {
 
 		return dirNamesVec[rf.dirs[i]] + fileNamesVec[rf.names[i]];
@@ -116,16 +129,6 @@ public:
 	void setCurContext(Domain* d) { curDomain = d; }
 	Domain* curContext() const { return curDomain; }
 
-	enum { // used as self-documenting boolean parameters
-		optFalse,
-		optSaveCache,
-		optGoDown,
-		optOnlyLoaded,
-		optDragDrop,
-		optFold,
-		optOnlyInIndex
-	};
-
 signals:
 	void newRevsAdded(const FileHistory*, const QValueVector<QString>&);
 	void loadCompleted(const FileHistory*, const QString&);
@@ -148,30 +151,54 @@ private:
 	friend class MainImpl;
 	friend class DataLoader;
 	friend class ConsoleImpl;
+	friend class RevsView;
 
-	typedef QMap<QString, QString> StrMap; // used to map SHA and patches file names
-	typedef QMap<QString, int> StrSet;     // for fast QString set lookup
+	struct Reference { // stores tag information associated to a revision
+		Reference() : type(0) {}
+		uint type;
+		QStringList branches;
+		QStringList remoteBranches;
+		QString     currentBranch;
+		QStringList tags;
+		QStringList refs;
+		QString     tagObj; // TODO support more then one obj
+		QString     tagMsg;
+		QString     stgitPatch;
+	};
+	typedef QMap<QString, Reference> RefMap;
+	RefMap refsShaMap;
+
+	struct WorkingDirInfo {
+		void clear() { diffIndex = diffIndexCached = ""; otherFiles.clear(); }
+		QString diffIndex;
+		QString diffIndexCached;
+		QStringList otherFiles;
+	};
+	WorkingDirInfo _wd;
 
 	bool run(SCRef cmd, QString* out = NULL, QObject* rcv = NULL, SCRef buf = "");
 	MyProcess* runAsync(SCRef cmd, QObject* rcv, SCRef buf = "");
 	MyProcess* runAsScript(SCRef cmd, QObject* rcv = NULL, SCRef buf = "");
 	bool allProcessDeleted();
-	const QString getArgs(QString* range, bool askForRange, bool* quit);
+	const QString getArgs(bool askForRange, bool* quit);
 	bool getRefs();
 	bool getStGITPatches();
-	bool lookUpPatchesSHA(SCRef pn, SList files, SList filesSHA, QString& sha);
-	void dirWalker(SCRef dirPath, SList files, SList filesSHA, SCRef nameFilter = "");
+	bool addStGitPatch(SCRef pn, SCList files, SCList filesSHA, bool applied);
+	static void dirWalker(SCRef dirPath, SList files, SList filesSHA, SCRef nameFilter = "");
 	void clearRevs();
-	bool startRevList(SCRef args, FileHistory* fh = NULL);
+	void clearFileNames();
+	bool startRevList(SCRef args, FileHistory* fh);
 	bool startUnappliedList();
 	bool startParseProc(SCRef initCmd, FileHistory* fh);
-	void addChunk(FileHistory* fh, SCRef parseBuffer);
+	int addChunk(FileHistory* fh, const QByteArray& ba, int ofs);
 	void parseDiffFormat(RevFile& rf, SCRef buf);
 	void parseDiffFormatLine(RevFile& rf, SCRef line, int parNum);
 	void getDiffIndex();
 	const QString getFileSha(SCRef file, SCRef revSha);
-	const Rev* fakeWorkDirRev(SCRef parent, SCRef log, SCRef longLog, int idx);
-	void copyDiffIndex(FileHistory* fh);
+	const Rev* fakeWorkDirRev(SCRef parent, SCRef log, SCRef longLog, int idx, FileHistory* fh);
+	const RevFile* fakeWorkDirRevFile(const WorkingDirInfo& wd);
+	bool copyDiffIndex(FileHistory* fh, SCRef parent);
+	const RevFile* insertNewFiles(SCRef sha, SCRef data);
 	const RevFile* getAllMergeFiles(const Rev* r);
 	bool isParentOf(SCRef par, SCRef child);
 	bool isTreeModified(SCRef sha);
@@ -181,43 +208,29 @@ private:
 	                   QMap<uint, QValueVector<int> >& dv);
 	void mergeNearTags(bool down, Rev* p, const Rev* r, const QMap<QPair<uint, uint>, bool>&dm);
 	void mergeBranches(Rev* p, const Rev* r);
-	void updateRefs(Rev& c, FileHistory* fh);
-	void updateLanes(Rev& c, Lanes& lns);
-	void removeFiles(SCList selFiles, SCRef workDir, SCRef ext);
-	void restoreFiles(SCList selFiles, SCRef workDir, SCRef ext);
+	void updateLanes(Rev& c, Lanes& lns, SCRef sha);
+	static void removeFiles(SCList selFiles, SCRef workDir, SCRef ext);
+	static void restoreFiles(SCList selFiles, SCRef workDir, SCRef ext);
 	bool mkPatchFromIndex(SCRef msg, SCRef patchFile);
 	const QStringList getOthersFiles();
 	const QStringList getOtherFiles(SCList selFiles, bool onlyInIndex);
-	const QString colorMatch(SCRef txt, QRegExp& regExp);
+	static const QString colorMatch(SCRef txt, QRegExp& regExp);
 	void appendFileName(RevFile& rf, SCRef name);
-	void populateFileDict();
-	const QString quote(SCRef nm);
-	const QString quote(SCList sl);
-	const QStringList noSpaceSepHack(SCRef cmd);
+	void populateFileNamesMap();
+	static const QString quote(SCRef nm);
+	static const QString quote(SCList sl);
+	static const QStringList noSpaceSepHack(SCRef cmd);
 	void removeDeleted(SCList selFiles);
-	void setStatus(RevFile& rf, SCRef stInfo, int parNum);
+	void setStatus(RevFile& rf, SCRef rowSt);
+	void setExtStatus(RevFile& rf, SCRef rowSt, int parNum);
+	void appendNamesWithId(QStringList& names, SCRef sha, SCList data, bool onlyLoaded);
+	Reference* lookupReference(SCRef sha, bool create = false);
 
 	EM_DECLARE(exGitStopped);
 
-	Cache* cache;
 	Domain* curDomain;
 	QString workDir; // workDir is always without trailing '/'
 	QString gitDir;
-	QStringList tags;
-	QStringList loadedTagNames;
-	QStringList loadedBranchNames;
-	QStringList tagsSHA;
-	StrMap tagsObj;
-	StrMap refsStillToFind; // to quick check if a rev is a ref during loading
-	StrMap refsStillToFindMasterCopy; // to restore original content for next search
-	QStringList heads;
-	QStringList headsSHA;
-	QStringList refs;
-	QStringList refsSHA;
-	QStringList unAppliedSHA;
-	QStringList appliedSHA;
-	StrMap patchNames;
-	QString currentBranchSHA;
 	QString filesLoadingPending;
 	QString filesLoadingCurSha;
 	QString curRange;
@@ -228,19 +241,15 @@ private:
 	bool isGIT;
 	bool isTextHighlighterFound;
 	bool loadingUnAppliedPatches;
-	bool unknownFiles;
-	bool nothingToCommit;
-	QString firstNonStGitPatch;
-	RevMap revs;
-	RevFileMap revsFiles;
-	Lanes lns;
+	bool fileCacheAccessed;
 	int patchesStillToFind;
+	QString firstNonStGitPatch;
+	RevFileMap revsFiles;
 	StrVect fileNamesVec;
 	StrVect dirNamesVec;
-	StrSet fileNames; // quick look-up file name
-	StrSet dirNames; // quick look-up dir name
-	StrVect revOrder;
-	uint firstFreeLane;
+	QMap<QString, int> fileNamesMap; // quick lookup file name
+	QMap<QString, int> dirNamesMap;  // quick lookup directory name
+	FileHistory revData;
 };
 
 #endif
