@@ -10,6 +10,7 @@
 #include <qapplication.h>
 #include <qheader.h>
 #include <qdragobject.h>
+#include <qdatetime.h>
 #include "common.h"
 #include "domain.h"
 #include "git.h"
@@ -25,8 +26,8 @@ ListView::ListView(Domain* dm, Git* g, QListView* l, FileHistory* f, const QFont
 	setupListView(fnt);
 	clear(); // to init some stuff
 
-	lv->setAcceptDrops(fh == NULL);
-	lv->viewport()->setAcceptDrops(fh == NULL);
+	lv->setAcceptDrops(git->isMainHistory(fh));
+	lv->viewport()->setAcceptDrops(git->isMainHistory(fh));
 	lv->viewport()->installEventFilter(this); // filter out some right clicks
 
 	connect(lv, SIGNAL(currentChanged(QListViewItem*)),
@@ -56,7 +57,7 @@ void ListView::setupListView(const QFont& fnt) {
 	lv->setSorting(-1);
 	lv->setFont(fnt);
 
-	int adj = fh ? 0 : -1;
+	int adj = !git->isMainHistory(fh) ? 0 : -1;
 
 	lv->setColumnWidthMode(GRAPH_COL, QListView::Manual);
 	lv->setColumnWidthMode(LOG_COL + adj, QListView::Manual);
@@ -84,7 +85,7 @@ void ListView::clear() {
 	lv->clear();
 	diffTarget = NULL; // avoid a dangling pointer
 
-	int adj = fh ? 0 : -1;
+	int adj = !git->isMainHistory(fh) ? 0 : -1;
 
 	if (testFlag(REL_DATE_F)) {
 		secs = QDateTime::currentDateTime().toTime_t();
@@ -97,7 +98,7 @@ void ListView::clear() {
 
 void ListView::updateIdValues() {
 
-	if (fh == NULL)
+	if (git->isMainHistory(fh))
 		return;
 
 	uint id = lv->childCount();
@@ -122,7 +123,7 @@ void ListView::getSelectedItems(QStringList& selectedItems) {
 
 const QString ListView::getSha(int id) {
 
-	if (fh == NULL)
+	if (git->isMainHistory(fh))
 		return "";
 
 	// check to early skip common case of list mouse browsing
@@ -199,7 +200,7 @@ bool ListView::update() {
 			lv->ensureItemVisible(item);
 		}
 	}
-	if (fh == NULL)
+	if (git->isMainHistory(fh))
 		setHighlight(st->diffToSha());
 
 	return (item != NULL);
@@ -209,7 +210,7 @@ bool ListView::update() {
 
 void ListView::on_newRevsAdded(const FileHistory* f, const QValueVector<QString>& shaVec) {
 
-	if (f != fh) // signal newRevsAdded() is broadcasted
+	if (f != fh) // signal newRevsAdded() is broadcast
 		return;
 
 	bool evenLine = !(lv->childCount() % 2);
@@ -217,10 +218,12 @@ void ListView::on_newRevsAdded(const FileHistory* f, const QValueVector<QString>
 	if (lv->childCount() == 0)
 		lastItem = NULL;
 
+	lv->setUpdatesEnabled(false);
 	for (uint i = lv->childCount(); i < shaVec.count(); i++) {
 		lastItem = new ListViewItem(lv, lastItem, git, shaVec[i], evenLine, secs, fh);
 		evenLine = !evenLine;
 	}
+	lv->setUpdatesEnabled(true);
 }
 
 void ListView::on_currentChanged(QListViewItem* item) {
@@ -310,14 +313,16 @@ bool ListView::filterRightButtonPressed(QMouseEvent* e) {
 			else
 				st->setDiffToSha(""); // restore std view
 
-			UPDATE_DOMAIN(d);
 			filterNextContextMenuRequest = true;
+			UPDATE_DOMAIN(d);
 			return true; // filter event out
 		}
 	}
 	// check for 'children & parents' function, i.e. if mouse is on the graph
 	int column = lv->header()->sectionAt(e->pos().x());
 	if (column == GRAPH_COL) {
+
+		filterNextContextMenuRequest = true;
 		QStringList parents, children;
 		if (getLaneParentsChilds(item, e->pos().x(), parents, children))
 			emit lanesContextMenuRequested(parents, children);
@@ -376,14 +381,14 @@ bool ListView::filterDropEvent(QDropEvent* e) {
 
 ListViewItem::ListViewItem(QListView* p, ListViewItem* a, Git* g, SCRef s,
               bool e, unsigned long t, FileHistory* f) : QListViewItem(p, a),
-              listView_(p), git(g), fh(f), sha_(s), secs(t), isEvenLine(e) {
+              listView_(p), git(g), fh(f), _sha(s), secs(t), isEvenLine(e) {
 
 	populated = isDiffTarget = isHighlighted = false;
 }
 
 int ListViewItem::getLaneType(uint pos) const {
 
-	const Rev* r = git->revLookup(sha_, fh); // 'r' cannot be NULL
+	const Rev* r = git->revLookup(_sha, fh); // 'r' cannot be NULL
 	return (pos < r->lanes.count() ? r->lanes[pos] : -1);
 }
 
@@ -541,7 +546,7 @@ void ListViewItem::paintGraph(const Rev& c, QPainter* p, const QColorGroup& cg,
 
 	const QValueVector<int>& lanes(c.lanes);
 	uint laneNum = lanes.count();
-	int mergeLane = -1;
+	uint mergeLane = 0;
 	for (uint i = 0; i < laneNum; i++)
 		if (isMerge(lanes[i])) {
 			mergeLane = i;
@@ -559,8 +564,8 @@ void ListViewItem::paintGraph(const Rev& c, QPainter* p, const QColorGroup& cg,
 		if (ln == EMPTY)
 			continue;
 
-		int col = (   isHead(ln) || isTail(ln) || isJoin(ln)
-		           || ln == CROSS_EMPTY) ? mergeLane : i;
+		uint col = (   isHead(ln) || isTail(ln) || isJoin(ln)
+		            || ln == CROSS_EMPTY) ? mergeLane : i;
 
 		if (ln == CROSS) {
 			paintGraphLane(p, NOT_ACTIVE, x1, x2, colors[col % COLORS_NUM], back);
@@ -573,7 +578,7 @@ void ListViewItem::paintGraph(const Rev& c, QPainter* p, const QColorGroup& cg,
 void ListViewItem::paintCell(QPainter* p, const QColorGroup& cg,
                              int column, int width, int alignment) {
 	QColorGroup _cg(cg);
-	const Rev& c = *git->revLookup(sha_, fh);
+	const Rev& c = *git->revLookup(_sha, fh);
 
 	// lazy setup, only once when visible
 	if (!populated) {
@@ -586,7 +591,7 @@ void ListViewItem::paintCell(QPainter* p, const QColorGroup& cg,
 	}
 
 	// adjust for annotation id column presence
-	int mycolumn = fh ? column : column + 1;
+	int mycolumn = (!git->isMainHistory(fh) ? column : column + 1);
 
 	// alternate background color
 	if (isInfoCol(mycolumn))
@@ -595,7 +600,7 @@ void ListViewItem::paintCell(QPainter* p, const QColorGroup& cg,
 	// tags, heads, refs and working dir colouring
 	if (mycolumn == LOG_COL) {
 
-		paintTagMarks(column, c);
+		paintTagMarks(column);
 
 		if (isHighlighted) {
 			QFont f(p->font());
@@ -620,22 +625,26 @@ void ListViewItem::paintCell(QPainter* p, const QColorGroup& cg,
 		p->restore();
 }
 
-void ListViewItem::paintTagMarks(int col, const Rev& c) {
+void ListViewItem::paintTagMarks(int col) {
 
-	if (!pixmap(col) && !c.isBranch && !c.isCurrentBranch && !c.isTag && !c.isRef)
+	uint rt = git->checkRef(_sha);
+
+	if (!pixmap(col) && rt == 0)
 		return; // common case
 
 	QPixmap* newPm = new QPixmap();
 
-	if (c.isBranch || c.isCurrentBranch) {
-		QColor color(c.isCurrentBranch ? Qt::green : DARK_GREEN);
-		addTextPixmap(&newPm, c.branch, color, c.isCurrentBranch);
-	}
-	if (c.isTag)
-		addTextPixmap(&newPm, c.tag, Qt::yellow);
+	if (rt & Git::BRANCH)
+		addBranchPixmap(&newPm);
 
-	if (c.isRef)
-		addTextPixmap(&newPm, c.ref, PURPLE);
+	if (rt & Git::RMT_BRANCH)
+		addRefPixmap(&newPm, git->getRefName(_sha, Git::RMT_BRANCH), LIGHT_ORANGE);
+
+	if (rt & Git::TAG)
+		addRefPixmap(&newPm, git->getRefName(_sha, Git::TAG), Qt::yellow);
+
+	if (rt & Git::REF)
+		addRefPixmap(&newPm, git->getRefName(_sha, Git::REF), PURPLE);
 
 	if (!pixmap(col) || (newPm->rect() != pixmap(col)->rect()))
 		setPixmap(col, *newPm);
@@ -643,14 +652,24 @@ void ListViewItem::paintTagMarks(int col, const Rev& c) {
 	delete newPm;
 }
 
-void ListViewItem::addTextPixmap(QPixmap** pp, SCRef text, const QColor& color, bool bold) {
+void ListViewItem::addBranchPixmap(QPixmap** pp) {
 
-	const QStringList sl(QStringList::split('\n', text));
-	loopList(it, sl)
-		doAddTextPixmap(pp, *it, color, bold);
+	QString curBranch;
+	SCList refs = git->getRefName(_sha, Git::BRANCH, &curBranch);
+	FOREACH_SL (it, refs) {
+		bool isCur = (curBranch == *it);
+		QColor color(isCur ? Qt::green : DARK_GREEN);
+		addTextPixmap(pp, *it, color, isCur);
+	}
 }
 
-void ListViewItem::doAddTextPixmap(QPixmap** pp, SCRef text, const QColor& color, bool bold) {
+void ListViewItem::addRefPixmap(QPixmap** pp, SCList refs, const QColor& color) {
+
+	FOREACH_SL (it, refs)
+		addTextPixmap(pp, *it, color, false);
+}
+
+void ListViewItem::addTextPixmap(QPixmap** pp, SCRef text, const QColor& color, bool bold) {
 
 	QFont fnt(myListView()->font());
 	if (bold)
@@ -686,8 +705,8 @@ bool ListViewItem::changedFiles(SCRef c) {
 
 	const RevFile* f = git->getFiles(c);
 	if (f)
-		for (uint i = 0; i < f->names.count(); i++)
-			if (!f->statusCmp(i, UNKNOWN))
+		for (int i = 0; i < f->count(); i++)
+			if (!f->statusCmp(i, RevFile::UNKNOWN))
 				return true;
 	return false;
 }
@@ -696,16 +715,16 @@ void ListViewItem::setupData(const Rev& c) {
 
 	// calculate lanes
 	if (c.lanes.count() == 0)
-		git->setLane(c.sha(), fh);
+		git->setLane(_sha, fh);
 
 	// set time/date column
-	int adj = fh ? 0 : -1;
-	if (c.sha() != ZERO_SHA) {
+	int adj = !git->isMainHistory(fh) ? 0 : -1;
+	if (_sha != ZERO_SHA) {
 		if (secs != 0) { // secs is 0 for absolute date
 			secs -= c.authorDate().toULong();
 			setText(TIME_COL + adj, timeDiff(secs));
 		} else
-			setText(TIME_COL + adj, git->getLocalDate(c.authorDate()));
+			setText(TIME_COL + adj, Git::getLocalDate(c.authorDate()));
 	}
 	setText(LOG_COL + adj, c.shortLog());
 	setText(AUTH_COL + adj, c.author());
