@@ -20,11 +20,10 @@ public:
 	ListViewItem(QListView* p, ListViewItem* a, Git* g, SCRef sha,
 	             bool e, unsigned long t, FileHistory* f);
 
-	SCRef sha() const { return sha_; }
+	SCRef sha() const { return _sha; }
 	int getLaneType(uint pos) const;
 	void setDiffTarget(bool b);
 	virtual void paintCell(QPainter* p, const QColorGroup& cg, int c, int w, int a);
-	SCRef getRefs();
 	void setEven(bool b) { isEvenLine = b; }
 	void setHighlighted(bool b) { isHighlighted = b; }
 	bool highlighted() const { return isHighlighted; }
@@ -34,9 +33,10 @@ private:
 	void setupData(const Rev& c);
 	void paintGraphLane(QPainter* p, int t, int x1, int x2, const QColor& c, const QBrush& b);
 	void paintGraph(const Rev& c, QPainter *p, const QColorGroup& cg, int width);
-	void paintTagMarks(int col, const Rev& c);
-	void addTextPixmap(QPixmap** pp, SCRef text, const QColor& color, bool bold = false);
-	void doAddTextPixmap(QPixmap** pp, SCRef text, const QColor& color, bool bold);
+	void paintTagMarks(int col);
+	void addBranchPixmap(QPixmap** pp);
+	void addRefPixmap(QPixmap** pp, SCList refs, const QColor& color);
+	void addTextPixmap(QPixmap** pp, SCRef text, const QColor& color, bool bold);
 	const QString timeDiff(unsigned long secs) const;
 	bool changedFiles(SCRef c);
 	QListView* myListView() const { return listView_; } // QListViewItem::listView() traverses the
@@ -44,7 +44,7 @@ private:
 	QListView* listView_;
 	Git* git;
 	FileHistory* fh;
-	const QString sha_;
+	const QString _sha;
 	unsigned long secs;
 	bool populated, isEvenLine, isHighlighted, isDiffTarget;
 };
