@@ -64,7 +64,7 @@ public:
 	// under some themes as Bluecurve or Platinum. What happens is that
 	// widget sizeHint() returns a wrong value that is used by Qt to paint
 	// the button.
-	// Becuase it is not possible to set sizeHint property directly,
+	// Because it is not possible to set sizeHint property directly,
 	// we need to sub class QPushButton and override this puppy
 	virtual QSize sizeHint() const {
 
@@ -221,11 +221,11 @@ void MainImpl::getExternalDiffArgs(QStringList* args) {
 
 	QString fileContent;
 	git->getFile(rv->st.fileName(), rv->st.sha(), NULL, &fileContent);
-	if (!git->writeToFile(fName1, fileContent))
+	if (!writeToFile(fName1, fileContent))
 		statusBar()->message("Unable to save " + fName1);
 
 	git->getFile(rv->st.fileName(), prevRevSha, NULL, &fileContent);
-	if (!git->writeToFile(fName2, fileContent))
+	if (!writeToFile(fName2, fileContent))
 		statusBar()->message("Unable to save " + fName2);
 
 	// get external diff viewer
@@ -365,11 +365,11 @@ void MainImpl::updateContextActions(SCRef newRevSha, SCRef newFileName,
 
 	if (found) {
 		const Rev* r = git->revLookup(newRevSha);
-		isTag = r->isTag;
+		isTag = git->checkRef(newRevSha, Git::TAG);
 		isUnApplied = r->isUnApplied;
 		isApplied = r->isApplied;
 	}
-	ActTag->setEnabled(found && !isTag && (newRevSha != ZERO_SHA) && !isUnApplied);
+	ActTag->setEnabled(found && (newRevSha != ZERO_SHA) && !isUnApplied);
 	ActTagDelete->setEnabled(found && isTag && (newRevSha != ZERO_SHA) && !isUnApplied);
 	ActPush->setEnabled(found && isUnApplied && git->isNothingToCommit());
 	ActPop->setEnabled(found && isApplied && git->isNothingToCommit());
@@ -507,7 +507,7 @@ bool MainImpl::eventFilter(QObject* obj, QEvent* ev) {
 
 void MainImpl::on_newRevsAdded(const FileHistory* fh, const QValueVector<QString>&) {
 
-	if (fh)
+	if (!git->isMainHistory(fh))
 		return;
 
 	if (toolButtonFilter->isOn())
@@ -531,13 +531,17 @@ void MainImpl::lineEditFilter_returnPressed() {
 void MainImpl::toolButtonFilter_toggled(bool isOn) {
 
 	toolButtonBold->setEnabled(!isOn);
-	filterList(isOn, false);
+	toolButtonFilter->setEnabled(false);
+	filterList(isOn, false); // blocking call
+	toolButtonFilter->setEnabled(true);
 }
 
 void MainImpl::toolButtonBold_toggled(bool isOn) {
 
 	toolButtonFilter->setEnabled(!isOn);
-	filterList(isOn, true);
+	toolButtonBold->setEnabled(false);
+	filterList(isOn, true); // blocking call
+	toolButtonBold->setEnabled(true);
 }
 
 void MainImpl::filterList(bool isOn, bool onlyHighlight) {
@@ -1050,10 +1054,13 @@ void MainImpl::doContexPopup(SCRef sha) {
 	// global scope because we use a signal/slot connection
 	delete contextMenu;
 	delete contextSubMenu;
+	delete contextRmtMenu;
 	contextMenu = new QPopupMenu(this);
 	contextSubMenu = new QPopupMenu(this);
+	contextRmtMenu = new QPopupMenu(this);
 	connect(contextMenu, SIGNAL(activated(int)), this, SLOT(on_goRef_activated(int)));
 	connect(contextSubMenu, SIGNAL(activated(int)), this, SLOT(on_goRef_activated(int)));
+	connect(contextRmtMenu, SIGNAL(activated(int)), this, SLOT(on_goRef_activated(int)));
 
 	Domain* t;
 	int tt = currentTabType(&t);
@@ -1091,20 +1098,29 @@ void MainImpl::doContexPopup(SCRef sha) {
 		if (ActPop->isEnabled())
 			ActPop->addTo(contextMenu);
 
-		const QStringList& bn(git->getBranchNames());
-		const QStringList& tn(git->getTagNames(Git::optOnlyLoaded));
-		if (bn.empty() && tn.empty()) {
+		const QStringList& bn(git->getAllRefNames(Git::BRANCH, Git::optOnlyLoaded));
+		const QStringList& rbn(git->getAllRefNames(Git::RMT_BRANCH, Git::optOnlyLoaded));
+		const QStringList& tn(git->getAllRefNames(Git::TAG, Git::optOnlyLoaded));
+		if (bn.empty() && rbn.empty() && tn.empty()) {
 			contextMenu->exec(QCursor::pos());
 			return;
 		}
-		int id = 1;
+		int id = 1; // ref names have id > 0 to disambiguate from actions
+		if (!rbn.empty()) {
+			QStringList::const_iterator it = rbn.constBegin();
+			for ( ; it != rbn.constEnd(); ++it, id++)
+				contextRmtMenu->insertItem(*it, id);
+		}
+		int rbnCnt = contextRmtMenu->count();
+		if (rbnCnt > 0)
+			contextMenu->insertItem("Remote branches", contextRmtMenu);
+
 		if (!bn.empty()) {
 			contextMenu->insertSeparator();
 			QStringList::const_iterator it = bn.constBegin();
 			for ( ; it != bn.constEnd(); ++it, id++) {
-				// branch names have id > 0 to disambiguate them from actions,
-				// Qt assigns negative id as default
-				if (id < MAX_MENU_ENTRIES)
+
+				if (id < MAX_MENU_ENTRIES + rbnCnt)
 					contextMenu->insertItem(*it, id);
 				else
 					contextSubMenu->insertItem(*it, id);
@@ -1114,9 +1130,8 @@ void MainImpl::doContexPopup(SCRef sha) {
 			contextMenu->insertSeparator();
 			QStringList::const_iterator it = tn.constBegin();
 			for ( ; it != tn.constEnd(); ++it, id++) {
-				// tag names have id > 0 to disambiguate them from actions,
-				// Qt assigns negative id as default
-				if (id < MAX_MENU_ENTRIES)
+
+				if (id < MAX_MENU_ENTRIES + rbnCnt)
 					contextMenu->insertItem(*it, id);
 				else
 					contextSubMenu->insertItem(*it, id);
@@ -1401,7 +1416,7 @@ void MainImpl::doUpdateCustomActionMenu(const QStringList& list) {
 		return;
 
 	Actions->insertSeparator();
-	loopList(it, list)
+	FOREACH_SL (it, list)
 		Actions->insertItem(*it);
 }
 
@@ -1496,7 +1511,7 @@ void MainImpl::ActTag_activated() {
 		             "are not allowed in tag name.");
 		return;
 	}
-	if (git->isTagName(tag)) {
+	if (!git->getRefSha(tag, Git::TAG, false).isEmpty()) {
 		QMessageBox::warning(this, "Make tag - QGit",
 		             "Sorry, tag name already exists.\n"
 		             "Please choose a different name.");
@@ -1535,7 +1550,7 @@ void MainImpl::ActPush_activated() {
 	QStringList selectedItems;
 	rv->listViewLog->getSelectedItems(selectedItems);
 	for (uint i = 0; i < selectedItems.count(); i++) {
-		if (!git->isUnapplied(selectedItems[i])) {
+		if (!git->checkRef(selectedItems[i], Git::UN_APPLIED)) {
 			statusBar()->message("Please, select only unapplied patches");
 			return;
 		}
@@ -1676,8 +1691,8 @@ void MainImpl::closeEvent(QCloseEvent* ce) {
 	// signal is connected to qApp->quit()
 	//
 	// note that we cannot rely on setting 'this' parent in secondary windows
-	// because when close() is called childern are still alive and, finally,
-	// when childern are deleted, d'tor do not call close() anymore. So we miss
+	// because when close() is called children are still alive and, finally,
+	// when children are deleted, d'tor do not call close() anymore. So we miss
 	// lastWindowClosed() signal in this case.
 	emit closeAllWindows();
 	hide();
@@ -1689,6 +1704,7 @@ void MainImpl::closeEvent(QCloseEvent* ce) {
 		// still some run() call not returned somewhere, it is
 		// not safe to delete run() callers objects now
 		QTimer::singleShot(100, this, SLOT(ActClose_activated()));
+		ce->ignore(); // just for sanity, already ignored by default
 		return;
 	}
 	emit closeAllTabs();
