@@ -11,6 +11,7 @@
 #include <qeventloop.h>
 #include "exceptionmanager.h"
 #include "common.h"
+#include "domain.h"
 #include "myprocess.h"
 
 MyProcess::MyProcess(QObject *go, Git* g, const QString& wd, bool err) : QProcess(g) {
@@ -51,6 +52,11 @@ bool MyProcess::runSync(SCRef rc, QString* ro, QObject* rcv, SCRef buf) {
 
 	busy = true; // we have to wait here until we exit
 
+	// we are now leaving our safe domain context and
+	// jump right in to processEvents() hyperspace
+	Domain* d = git->curContext();
+	git->setCurContext(NULL);
+
 	EM_BEFORE_PROCESS_EVENTS;
 
 	while (busy) {
@@ -64,6 +70,12 @@ bool MyProcess::runSync(SCRef rc, QString* ro, QObject* rcv, SCRef buf) {
 
 	EM_AFTER_PROCESS_EVENTS;
 
+	if (git->curContext() != NULL)
+		qDebug("ASSERT in MyProcess::runSync, context is %p "
+		       "instead of NULL", (void*)git->curContext());
+
+	git->setCurContext(d); // restore our context
+
 	return exitStatus;
 }
 
@@ -78,6 +90,9 @@ void MyProcess::setupSignals() {
 		        receiver, SLOT(on_procDataReady(const QString&)));
 		connect(this, SIGNAL(eof()), receiver, SLOT(on_eof()));
 	}
+	Domain* d = git->curContext();
+	if (d)
+		connect(d, SIGNAL(cancelDomainProcesses()), this, SLOT(on_cancel()));
 }
 
 void MyProcess::sendErrorMsg(bool notStarted) {
@@ -99,12 +114,10 @@ bool MyProcess::launchMe(SCRef runCmd, SCRef buf) {
 	const QStringList sl(splitArgList(runCmd));
 	setArguments(sl);
 	setWorkingDirectory(workDir);
-	git->incRunningProcesses();
 	bool ok = launch(buf);
-	if (!ok) {
-		git->decRunningProcesses();
+	if (!ok)
 		sendErrorMsg(true);
-	}
+
 	return ok;
 }
 
@@ -126,36 +139,34 @@ void MyProcess::on_readyReadStdout() {
 
 void MyProcess::on_readyReadStderr() {
 
-	if (receiver == NULL) {
-		dbs("ASSERT in myReadFromStderr: NULL receiver");
-		return;
-	}
 	const QString tmp(readStderr());
 	if (canceling)
 		return;
 
-	emit procDataReady(tmp); // redirect to stdout
+	if (receiver != NULL)
+		emit procDataReady(tmp); // redirect to stdout
+	else {
+		dbs("ASSERT in myReadFromStderr: NULL receiver");
+		return;
+	}
 }
 
 void MyProcess::on_processExited() {
 
+	if (canceling || !normalExit() || canReadLineStderr())
+		exitStatus = false;
+
 	if (!canceling) { // no more noise after cancel
 
 		if (receiver)
 			emit eof();
 
-		if (!normalExit() || canReadLineStderr()) {
-			exitStatus = false;
+		if (!exitStatus)
 			sendErrorMsg();
-		}
 	}
 	busy = false;
 	if (async)
 		deleteLater();
-
-	// must be here, in event handler not in d'tor to avoid a
-	// deadlock in case git is stopped in the meanwhile.
-	git->decRunningProcesses();
 }
 
 void MyProcess::on_cancel() {
@@ -164,6 +175,11 @@ void MyProcess::on_cancel() {
 	QProcess::tryTerminate();
 }
 
+void MyProcess::parseArgs(SCRef cmd, SList args) {
+
+	args = splitArgList(cmd);
+}
+
 const QStringList MyProcess::splitArgList(SCRef cmd) {
 // return argument list handling quotes and double quotes
 // substring, as example from:
@@ -205,7 +221,7 @@ const QStringList MyProcess::splitArgList(SCRef cmd) {
 
 	// QProcess::setArguments doesn't want quote
 	// delimited arguments, so remove trailing quotes
-	QStringList sl = QStringList::split(sepChar, newCmd);
+	QStringList sl(QStringList::split(sepChar, newCmd));
 	QStringList::iterator it(sl.begin());
 	for ( ; it != sl.end(); ++it) {
 		if (((*it).left(1) == "\"" && (*it).right(1) == "\"") ||
