@@ -46,12 +46,13 @@ public slots:
 	void on_procDataReady(const QString& data);
 	void on_eof();
 	void on_highlightPatch(const QString&, bool);
+	void on_updateRevDesc();
 
 protected slots:
 	virtual void on_contextMenu(const QString&, int);
 
 protected:
-	virtual bool doUpdate();
+	virtual bool doUpdate(bool force);
 
 private:
 	friend class DiffHighlighter;
@@ -77,6 +78,12 @@ private:
 	bool isRegExp;
 	QRegExp pickAxeRE;
 
+	enum ButtonId {
+		DIFF_TO_PARENT = 0,
+		DIFF_TO_HEAD   = 1,
+		DIFF_TO_SHA    = 2
+	};
+
 	struct MatchSelection {
 		int paraFrom;
 		int indexFrom;
