@@ -41,7 +41,7 @@ RevsView::RevsView(MainImpl* mi, Git* g) : Domain(mi, g) {
 	m()->tabWdg->addTab(revTab, "&Rev list");
 	tabPosition = m()->tabWdg->count() - 1;
 
-	listViewLog = new ListView(this, git, tab()->listViewLog, NULL, m()->listViewFont);
+	listViewLog = new ListView(this, g, tab()->listViewLog, &(g->revData), m()->listViewFont);
 	tab()->textBrowserDesc->setDomain(this);
 	listBoxFiles = new ListBoxFiles(this, git, tab()->listBoxFiles);
 	treeView = new TreeView(this, git, m()->treeView);
@@ -132,7 +132,7 @@ void RevsView::viewPatch(bool newTab) {
 
 void RevsView::on_loadCompleted(const FileHistory* fh, const QString& stats) {
 
-	if (fh)
+	if (!git->isMainHistory(fh))
 		return;
 
 	if (st.sha().isEmpty()) { // point to first one in list
@@ -172,13 +172,14 @@ bool RevsView::doUpdate(bool force) {
 
 			updateLineEditSHA();
 			on_updateRevDesc();
-			m()->statusBar()->message(git->getRevInfo(st.sha(), false));
+			m()->statusBar()->message(git->getRevInfo(st.sha()));
 		}
 		const RevFile* files = NULL;
 		bool newFiles = false;
 
 		if (st.isChanged(StateInfo::ANY & ~StateInfo::FILE_NAME) || force) {
 
+			listBoxFiles->clear();
 			files = git->getFiles(st.sha(), st.diffToSha(), st.allMergeFiles());
 			newFiles = true;
 		}
@@ -240,14 +241,14 @@ void RevsView::on_lanesContextMenuRequested(SCList parents, SCList childs) {
 
 	QPopupMenu contextMenu;
 	uint i = 0;
-	loopList(c, childs)
-		contextMenu.insertItem("Child: " + git->getShortLog(*c), i++);
+	FOREACH_SL (it, childs)
+		contextMenu.insertItem("Child: " + git->getShortLog(*it), i++);
 
-	loopList(p, parents) {
+	FOREACH_SL (it, parents) {
 
-		QString log(git->getShortLog(*p));
+		QString log(git->getShortLog(*it));
 		if (log.isEmpty())
-			log = *p;
+			log = *it;
 
 		contextMenu.insertItem("Parent: " + log, i++);
 	}
