@@ -227,30 +227,19 @@ void TreeView::setMimePix(SCRef ext, QPixmap* pix) {
 		mimePixMap.insert(ext, pix);
 }
 
-QPixmap* TreeView::mimePix(SCRef fileName) {
+const QPixmap* TreeView::mimePix(SCRef fileName) {
 
 	SCRef ext = fileName.section('.', -1, -1);
-	if (ext.isEmpty())
-		return fileDefault;
-
 	QPixmap* pix = mimePixMap.find(ext);
 	return (pix ? pix : fileDefault);
 }
 
 void TreeView::setTree(SCRef treeSha) {
 
-	if (listView->childCount() == 0) {
-
+	if (listView->childCount() == 0)
 		// get working dir info only once after each TreeView::clear()
-		git->getWorkDirFiles(MODIFIED, modifiedFiles, modifiedDirs);
-		QStringList f, d;
-		git->getWorkDirFiles(DELETED, f, d);
-		modifiedFiles += f;
-		modifiedDirs += d;
-		git->getWorkDirFiles(UNKNOWN, f, d);
-		modifiedFiles += f;
-		modifiedDirs += d;
-	}
+		git->getWorkDirFiles(modifiedFiles, modifiedDirs, RevFile::ANY);
+
 	listView->clear();
 	treeIsValid = true;
 
@@ -283,9 +272,14 @@ void TreeView::update() {
 	    && root->treeSha != ZERO_SHA)
 		// root->treeSha could reference a different sha from current
 		// one in case the tree is the same, i.e. has the same files.
-		// so we must use the state sha to call isSameFiles() and
-		// benefit from the early skip logic
-		newTree = !git->isSameFiles(st->sha(false), st->sha(true));
+		// so we prefer to use the previous state sha to call isSameFiles()
+		// and benefit from the early skip logic.
+		// In case previous sha is the same of current it means an update
+		// call has been forced, in that case we use the 'real' root->treeSha
+		if (st->sha(true) != st->sha(false))
+			newTree = !git->isSameFiles(st->sha(false), st->sha(true));
+		else
+			newTree = !git->isSameFiles(root->treeSha, st->sha(true));
 
 	if (newTree) // ok, we really need to update the tree
 		setTree(st->sha());
@@ -305,9 +299,9 @@ void TreeView::update() {
 	const QStringList lst(QStringList::split("/", st->fileName()));
 	QListViewItem* item = listView->firstChild();
 	item = item->itemBelow(); // first item is repository name
-	loopList(it2, lst) {
+	FOREACH_SL (it, lst) {
 		while (item && treeIsValid) {
-			if (item->text(0) == *it2) {
+			if (item->text(0) == *it) {
 				// could be a different subdirectory with the
 				// same name that appears before in tree view
 				// to be sure we need to check the names
@@ -325,7 +319,7 @@ void TreeView::update() {
 		listView->setCurrentItem(item); // calls on_currentChanged()
 		listView->ensureItemVisible(item);
 	} else
-		; // st->fileName() has beed deleted by a patch older then this tree
+		; // st->fileName() has been deleted by a patch older than this tree
 
 	listView->setUpdatesEnabled(true);
 	listView->triggerUpdate();
